# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from concurrent.futures import ThreadPoolExecutor
import logging
import threading
from typing import Iterator

import grpc

import phone_pb2
import phone_pb2_grpc


class CallMaker:
    def __init__(
        self,
        executor: ThreadPoolExecutor,
        channel: grpc.Channel,
        phone_number: str,
    ) -> None:
        self._executor = executor
        self._channel = channel
        self._stub = phone_pb2_grpc.PhoneStub(self._channel)
        self._phone_number = phone_number
        self._session_id = None
        self._audio_session_link = None
        self._call_state = None
        self._peer_responded = threading.Event()
        self._call_finished = threading.Event()
        self._consumer_future = None

    def _response_watcher(
        self, response_iterator: Iterator[phone_pb2.StreamCallResponse]
    ) -> None:
        try:
            for response in response_iterator:
                # NOTE: All fields in Proto3 are optional. This is the recommended way
                # to check if a field is present or not, or to exam which one-of field is
                # fulfilled by this message.
                if response.HasField("call_info"):
                    self._on_call_info(response.call_info)
                elif response.HasField("call_state"):
                    self._on_call_state(response.call_state.state)
                else:
                    raise RuntimeError(
                        "Received StreamCallResponse without call_info and"
                        " call_state"
                    )
        except Exception as e:
            self._peer_responded.set()
            raise

    def _on_call_info(self, call_info: phone_pb2.CallInfo) -> None:
        self._session_id = call_info.session_id
        self._audio_session_link = call_info.media

    def _on_call_state(self, call_state: phone_pb2.CallState.State) -> None:
        logging.info(
            "Call toward [%s] enters [%s] state",
            self._phone_number,
            phone_pb2.CallState.State.Name(call_state),
        )
        self._call_state = call_state
        if call_state == phone_pb2.CallState.State.ACTIVE:
            self._peer_responded.set()
        if call_state == phone_pb2.CallState.State.ENDED:
            self._peer_responded.set()
            self._call_finished.set()

    def call(self) -> None:
        request = phone_pb2.StreamCallRequest()
        request.phone_number = self._phone_number
        response_iterator = self._stub.StreamCall(iter((request,)))
        # Instead of consuming the response on current thread, spawn a consumption thread.
        self._consumer_future = self._executor.submit(
            self._response_watcher, response_iterator
        )

    def wait_peer(self) -> bool:
        logging.info("Waiting for peer to connect [%s]...", self._phone_number)
        self._peer_responded.wait(timeout=None)
        if self._consumer_future.done():
            # If the future raises, forwards the exception here
            self._consumer_future.result()
        return self._call_state == phone_pb2.CallState.State.ACTIVE

    def audio_session(self) -> None:
        assert self._audio_session_link is not None
        logging.info("Consuming audio resource [%s]", self._audio_session_link)
        self._call_finished.wait(timeout=None)
        logging.info("Audio session finished [%s]", self._audio_session_link)


def process_call(
    executor: ThreadPoolExecutor, channel: grpc.Channel, phone_number: str
) -> None:
    call_maker = CallMaker(executor, channel, phone_number)
    call_maker.call()
    if call_maker.wait_peer():
        call_maker.audio_session()
        logging.info("Call finished!")
    else:
        logging.info("Call failed: peer didn't answer")


def run():
    executor = ThreadPoolExecutor()
    with grpc.insecure_channel("localhost:50051") as channel:
        future = executor.submit(
            process_call, executor, channel, "555-0100-XXXX"
        )
        future.result()


if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    run()



#!/usr/bin/env python3

# Copyright 2023 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# USAGE:
# Run some tests with the GRPC_ARENA_TRACE_POOLED_ALLOCATIONS #define turned on.
# Capture the output to a text file.
# Invoke this program with that as an argument, and let it work its magic.

import collections
import heapq
import random
import re
import sys

# A single allocation, negative size => free
Allocation = collections.namedtuple("Allocation", "size ptr")
Active = collections.namedtuple("Active", "id size")

# Read through all the captures, and build up scrubbed traces
arenas = []
building = collections.defaultdict(list)
active = {}
biggest = 0
smallest = 1024
sizes = set()
for filename in sys.argv[1:]:
    for line in open(filename):
        m = re.search(
            r"ARENA 0x([0-9a-f]+) ALLOC ([0-9]+) @ 0x([0-9a-f]+)", line
        )
        if m:
            size = int(m.group(2))
            if size > biggest:
                biggest = size
            if size < smallest:
                smallest = size
            active[m.group(3)] = Active(m.group(1), size)
            building[m.group(1)].append(size)
            sizes.add(size)
        m = re.search(r"FREE 0x([0-9a-f]+)", line)
        if m:
            # We may have spurious frees, so make sure there's an outstanding allocation
            last = active.pop(m.group(1), None)
            if last is not None:
                building[last.id].append(-last.size)
        m = re.search(r"DESTRUCT_ARENA 0x([0-9a-f]+)", line)
        if m:
            trace = building.pop(m.group(1), None)
            if trace:
                arenas.append(trace)


# Given a list of pool sizes, return which bucket an allocation should go into
def bucket(pool_sizes, size):
    for bucket in sorted(pool_sizes):
        if abs(size) <= bucket:
            return bucket


# Given a list of pool sizes, determine the total outstanding bytes in the arena for once trace
def outstanding_bytes(pool_sizes, trace):
    free_list = collections.defaultdict(int)
    allocated = 0
    for size in trace:
        b = bucket(pool_sizes, size)
        if size < 0:
            free_list[b] += 1
        else:
            if free_list[b] > 0:
                free_list[b] -= 1
            else:
                allocated += b
    return allocated + len(pool_sizes) * 8


# Given a list of pool sizes, determine the maximum outstanding bytes for any seen trace
def measure(pool_sizes):
    max_outstanding = 0
    for trace in arenas:
        max_outstanding = max(
            max_outstanding, outstanding_bytes(pool_sizes, trace)
        )
    return max_outstanding


ALWAYS_INCLUDE = 1024
best = [ALWAYS_INCLUDE, biggest]
best_measure = measure(best)

testq = []
step = 0


def add(l):
    global testq, best_measure, best
    m = measure(l)
    if m < best_measure:
        best_measure = m
        best = l
    if l[-1] == smallest:
        return
    heapq.heappush(testq, (m, l))


add(best)

while testq:
    top = heapq.heappop(testq)[1]
    m = measure(top)
    step += 1
    if step % 1000 == 0:
        print(
            "iter %d; pending=%d; top=%r/%d"
            % (step, len(testq), top, measure(top))
        )
    for i in sizes:
        if i >= top[-1]:
            continue
        add(top + [i])

print("SAW SIZES: %r" % sorted(list(sizes)))
print("BEST: %r" % list(reversed(best)))
print("BEST MEASURE: %d" % best_measure)



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests of grpc_reflection.v1alpha.reflection."""

import logging
import unittest

from google.protobuf import descriptor_pb2
import grpc
from grpc.experimental import aio
from grpc_reflection.v1alpha import reflection
from grpc_reflection.v1alpha import reflection_pb2
from grpc_reflection.v1alpha import reflection_pb2_grpc

from src.proto.grpc.testing import empty_pb2
from src.proto.grpc.testing.proto2 import empty2_extensions_pb2
from src.proto.grpc.testing.proto2 import empty2_pb2
from tests_aio.unit._test_base import AioTestBase

_EMPTY_PROTO_FILE_NAME = "src/proto/grpc/testing/empty.proto"
_EMPTY_PROTO_SYMBOL_NAME = "grpc.testing.Empty"
_SERVICE_NAMES = (
    "Angstrom",
    "Bohr",
    "Curie",
    "Dyson",
    "Einstein",
    "Feynman",
    "Galilei",
)
_EMPTY_EXTENSIONS_SYMBOL_NAME = "grpc.testing.proto2.EmptyWithExtensions"
_EMPTY_EXTENSIONS_NUMBERS = (
    124,
    125,
    126,
    127,
    128,
)


def _file_descriptor_to_proto(descriptor):
    proto = descriptor_pb2.FileDescriptorProto()
    descriptor.CopyToProto(proto)
    return proto.SerializeToString()


class ReflectionServicerTest(AioTestBase):
    async def setUp(self):
        self._server = aio.server()
        reflection.enable_server_reflection(_SERVICE_NAMES, self._server)
        port = self._server.add_insecure_port("[::]:0")
        await self._server.start()

        self._channel = aio.insecure_channel("localhost:%d" % port)
        self._stub = reflection_pb2_grpc.ServerReflectionStub(self._channel)

    async def tearDown(self):
        await self._server.stop(None)
        await self._channel.close()

    async def test_file_by_name(self):
        requests = (
            reflection_pb2.ServerReflectionRequest(
                file_by_filename=_EMPTY_PROTO_FILE_NAME
            ),
            reflection_pb2.ServerReflectionRequest(
                file_by_filename="i-donut-exist"
            ),
        )
        responses = []
        async for response in self._stub.ServerReflectionInfo(iter(requests)):
            responses.append(response)
        expected_responses = (
            reflection_pb2.ServerReflectionResponse(
                valid_host="",
                file_descriptor_response=reflection_pb2.FileDescriptorResponse(
                    file_descriptor_proto=(
                        _file_descriptor_to_proto(empty_pb2.DESCRIPTOR),
                    )
                ),
                original_request=requests[0],
            ),
            reflection_pb2.ServerReflectionResponse(
                valid_host="",
                error_response=reflection_pb2.ErrorResponse(
                    error_code=grpc.StatusCode.NOT_FOUND.value[0],
                    error_message=grpc.StatusCode.NOT_FOUND.value[1].encode(),
                ),
                original_request=requests[1],
            ),
        )
        self.assertSequenceEqual(expected_responses, responses)

    async def test_file_by_symbol(self):
        requests = (
            reflection_pb2.ServerReflectionRequest(
                file_containing_symbol=_EMPTY_PROTO_SYMBOL_NAME
            ),
            reflection_pb2.ServerReflectionRequest(
                file_containing_symbol="i.donut.exist.co.uk.org.net.me.name.foo"
            ),
        )
        responses = []
        async for response in self._stub.ServerReflectionInfo(iter(requests)):
            responses.append(response)
        expected_responses = (
            reflection_pb2.ServerReflectionResponse(
                valid_host="",
                file_descriptor_response=reflection_pb2.FileDescriptorResponse(
                    file_descriptor_proto=(
                        _file_descriptor_to_proto(empty_pb2.DESCRIPTOR),
                    )
                ),
                original_request=requests[0],
            ),
            reflection_pb2.ServerReflectionResponse(
                valid_host="",
                error_response=reflection_pb2.ErrorResponse(
                    error_code=grpc.StatusCode.NOT_FOUND.value[0],
                    error_message=grpc.StatusCode.NOT_FOUND.value[1].encode(),
                ),
                original_request=requests[1],
            ),
        )
        self.assertSequenceEqual(expected_responses, responses)

    async def test_file_containing_extension(self):
        requests = (
            reflection_pb2.ServerReflectionRequest(
                file_containing_extension=reflection_pb2.ExtensionRequest(
                    containing_type=_EMPTY_EXTENSIONS_SYMBOL_NAME,
                    extension_number=125,
                ),
            ),
            reflection_pb2.ServerReflectionRequest(
                file_containing_extension=reflection_pb2.ExtensionRequest(
                    containing_type="i.donut.exist.co.uk.org.net.me.name.foo",
                    extension_number=55,
                ),
            ),
        )
        responses = []
        async for response in self._stub.ServerReflectionInfo(iter(requests)):
            responses.append(response)
        expected_responses = (
            reflection_pb2.ServerReflectionResponse(
                valid_host="",
                file_descriptor_response=reflection_pb2.FileDescriptorResponse(
                    file_descriptor_proto=(
                        _file_descriptor_to_proto(
                            empty2_extensions_pb2.DESCRIPTOR
                        ),
                        _file_descriptor_to_proto(empty2_pb2.DESCRIPTOR),
                    )
                ),
                original_request=requests[0],
            ),
            reflection_pb2.ServerReflectionResponse(
                valid_host="",
                error_response=reflection_pb2.ErrorResponse(
                    error_code=grpc.StatusCode.NOT_FOUND.value[0],
                    error_message=grpc.StatusCode.NOT_FOUND.value[1].encode(),
                ),
                original_request=requests[1],
            ),
        )
        self.assertSequenceEqual(expected_responses, responses)

    async def test_extension_numbers_of_type(self):
        requests = (
            reflection_pb2.ServerReflectionRequest(
                all_extension_numbers_of_type=_EMPTY_EXTENSIONS_SYMBOL_NAME
            ),
            reflection_pb2.ServerReflectionRequest(
                all_extension_numbers_of_type="i.donut.exist.co.uk.net.name.foo"
            ),
        )
        responses = []
        async for response in self._stub.ServerReflectionInfo(iter(requests)):
            responses.append(response)
        expected_responses = (
            reflection_pb2.ServerReflectionResponse(
                valid_host="",
                all_extension_numbers_response=reflection_pb2.ExtensionNumberResponse(
                    base_type_name=_EMPTY_EXTENSIONS_SYMBOL_NAME,
                    extension_number=_EMPTY_EXTENSIONS_NUMBERS,
                ),
                original_request=requests[0],
            ),
            reflection_pb2.ServerReflectionResponse(
                valid_host="",
                error_response=reflection_pb2.ErrorResponse(
                    error_code=grpc.StatusCode.NOT_FOUND.value[0],
                    error_message=grpc.StatusCode.NOT_FOUND.value[1].encode(),
                ),
                original_request=requests[1],
            ),
        )
        self.assertSequenceEqual(expected_responses, responses)

    async def test_list_services(self):
        requests = (
            reflection_pb2.ServerReflectionRequest(
                list_services="",
            ),
        )
        responses = []
        async for response in self._stub.ServerReflectionInfo(iter(requests)):
            responses.append(response)
        expected_responses = (
            reflection_pb2.ServerReflectionResponse(
                valid_host="",
                list_services_response=reflection_pb2.ListServiceResponse(
                    service=tuple(
                        reflection_pb2.ServiceResponse(name=name)
                        for name in _SERVICE_NAMES
                    )
                ),
                original_request=requests[0],
            ),
        )
        self.assertSequenceEqual(expected_responses, responses)

    def test_reflection_service_name(self):
        self.assertEqual(
            reflection.SERVICE_NAME, "grpc.reflection.v1alpha.ServerReflection"
        )


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main(verbosity=2)



#!/usr/bin/env python3
# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Starts a local DNS server for use in tests"""

import argparse
import os
import platform
import signal
import sys
import threading
import time

import twisted
import twisted.internet
import twisted.internet.defer
import twisted.internet.protocol
import twisted.internet.reactor
import twisted.internet.threads
import twisted.names
from twisted.names import authority
from twisted.names import client
from twisted.names import common
from twisted.names import dns
from twisted.names import server
import twisted.names.client
import twisted.names.dns
import twisted.names.server
import yaml

_SERVER_HEALTH_CHECK_RECORD_NAME = (  # missing end '.' for twisted syntax
    "health-check-local-dns-server-is-alive.resolver-tests.grpctestingexp"
)
_SERVER_HEALTH_CHECK_RECORD_DATA = "123.123.123.123"

_shutdown_event = threading.Event()
_thread_sleep_lock = threading.Lock()
_SLEEP_TIME = 1.0


class NoFileAuthority(authority.FileAuthority):
    def __init__(self, soa, records):
        # skip FileAuthority
        common.ResolverBase.__init__(self)
        self.soa = soa
        self.records = records


def start_local_dns_server(args):
    all_records = {}

    def _push_record(name, r):
        name = name.encode("ascii")
        print("pushing record: |%s|" % name)
        if all_records.get(name) is not None:
            all_records[name].append(r)
            return
        all_records[name] = [r]

    def _maybe_split_up_txt_data(name, txt_data, r_ttl):
        txt_data = txt_data.encode("ascii")
        start = 0
        txt_data_list = []
        while len(txt_data[start:]) > 0:
            next_read = len(txt_data[start:])
            if next_read > 255:
                next_read = 255
            txt_data_list.append(txt_data[start : start + next_read])
            start += next_read
        _push_record(name, dns.Record_TXT(*txt_data_list, ttl=r_ttl))

    with open(args.records_config_path) as config:
        test_records_config = yaml.safe_load(config)
    common_zone_name = test_records_config["resolver_tests_common_zone_name"]
    for group in test_records_config["resolver_component_tests"]:
        for name in group["records"].keys():
            for record in group["records"][name]:
                r_type = record["type"]
                r_data = record["data"]
                r_ttl = int(record["TTL"])
                record_full_name = "%s.%s" % (name, common_zone_name)
                assert record_full_name[-1] == "."
                record_full_name = record_full_name[:-1]
                if r_type == "A":
                    _push_record(
                        record_full_name, dns.Record_A(r_data, ttl=r_ttl)
                    )
                if r_type == "AAAA":
                    _push_record(
                        record_full_name, dns.Record_AAAA(r_data, ttl=r_ttl)
                    )
                if r_type == "SRV":
                    p, w, port, target = r_data.split(" ")
                    p = int(p)
                    w = int(w)
                    port = int(port)
                    target_full_name = (
                        "%s.%s" % (target, common_zone_name)
                    ).encode("ascii")
                    _push_record(
                        record_full_name,
                        dns.Record_SRV(p, w, port, target_full_name, ttl=r_ttl),
                    )
                if r_type == "TXT":
                    _maybe_split_up_txt_data(record_full_name, r_data, r_ttl)
    # Add an optional IPv4 record is specified
    if args.add_a_record:
        extra_host, extra_host_ipv4 = args.add_a_record.split(":")
        _push_record(extra_host, dns.Record_A(extra_host_ipv4, ttl=0))
    # Server health check record
    _push_record(
        _SERVER_HEALTH_CHECK_RECORD_NAME,
        dns.Record_A(_SERVER_HEALTH_CHECK_RECORD_DATA, ttl=0),
    )
    soa_record = dns.Record_SOA(mname=common_zone_name.encode("ascii"))
    test_domain_com = NoFileAuthority(
        soa=(common_zone_name.encode("ascii"), soa_record),
        records=all_records,
    )
    server = twisted.names.server.DNSServerFactory(
        authorities=[test_domain_com], verbose=2
    )
    server.noisy = 2
    twisted.internet.reactor.listenTCP(args.port, server)
    dns_proto = twisted.names.dns.DNSDatagramProtocol(server)
    dns_proto.noisy = 2
    twisted.internet.reactor.listenUDP(args.port, dns_proto)
    print("starting local dns server on 127.0.0.1:%s" % args.port)
    print("starting twisted.internet.reactor")
    twisted.internet.reactor.suggestThreadPoolSize(1)
    twisted.internet.reactor.run()


def _quit_on_signal(signum, _frame):
    print("Received SIGNAL %d. Quitting with exit code 0" % signum)
    # Allow the flush thread to exit gracefully.
    # This prevents a data race when the flush thread is sleeping and the main
    # thread calls sys.exit(). See b/474047558.
    _shutdown_event.set()
    # Instead of just sleeping for _SLEEP_TIME, to exit faster we use a lock
    # to signal from the flush thread that it's ready to exit.
    _thread_sleep_lock.acquire(timeout=_SLEEP_TIME + 0.1)

    twisted.internet.reactor.stop()
    sys.stdout.flush()
    sys.exit(0)


def flush_stdout_loop():
    num_timeouts_so_far = 0
    # Prevent zombies. Tests that use this server are short-lived.
    max_timeouts = 60 * 10
    while num_timeouts_so_far < max_timeouts:
        if _shutdown_event.is_set():
            # Allow the flush thread to exit gracefully to prevent data races
            # with the main thread during shutdown.
            print("Shutting down the stdout flush thread.", flush=True)
            return

        sys.stdout.flush()
        with _thread_sleep_lock:
            time.sleep(_SLEEP_TIME)
        num_timeouts_so_far += 1
    print("Process timeout reached, or cancelled. Exiting 0.")
    os.kill(os.getpid(), signal.SIGTERM)


def main():
    argp = argparse.ArgumentParser(
        description="Local DNS Server for resolver tests"
    )
    argp.add_argument(
        "-p",
        "--port",
        default=None,
        type=int,
        help="Port for DNS server to listen on for TCP and UDP.",
    )
    argp.add_argument(
        "-r",
        "--records_config_path",
        default=None,
        type=str,
        help=(
            "Directory of resolver_test_record_groups.yaml file. "
            "Defaults to path needed when the test is invoked as part "
            "of run_tests.py."
        ),
    )
    argp.add_argument(
        "--add_a_record",
        default=None,
        type=str,
        help=(
            "Add an A record via the command line. Useful for when we "
            "need to serve a one-off A record that is under a "
            "different domain then the rest the records configured in "
            "--records_config_path (which all need to be under the "
            "same domain). Format: <name>:<ipv4 address>"
        ),
    )
    args = argp.parse_args()
    signal.signal(signal.SIGTERM, _quit_on_signal)
    signal.signal(signal.SIGINT, _quit_on_signal)
    output_flush_thread = threading.Thread(target=flush_stdout_loop)
    output_flush_thread.setDaemon(True)
    output_flush_thread.start()
    start_local_dns_server(args)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/admin/v3/memory.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/admin/v3/memory.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x65nvoy/admin/v3/memory.proto\x12\x0e\x65nvoy.admin.v3\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\"\xbd\x01\n\x06Memory\x12\x11\n\tallocated\x18\x01 \x01(\x04\x12\x11\n\theap_size\x18\x02 \x01(\x04\x12\x19\n\x11pageheap_unmapped\x18\x03 \x01(\x04\x12\x15\n\rpageheap_free\x18\x04 \x01(\x04\x12\x1a\n\x12total_thread_cache\x18\x05 \x01(\x04\x12\x1c\n\x14total_physical_bytes\x18\x06 \x01(\x04:!\x9a\xc5\x88\x1e\x1c\n\x1a\x65nvoy.admin.v2alpha.MemoryBt\n\x1cio.envoyproxy.envoy.admin.v3B\x0bMemoryProtoP\x01Z=github.com/envoyproxy/go-control-plane/envoy/admin/v3;adminv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.admin.v3.memory_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034io.envoyproxy.envoy.admin.v3B\013MemoryProtoP\001Z=github.com/envoyproxy/go-control-plane/envoy/admin/v3;adminv3\272\200\310\321\006\002\020\002'
  _globals['_MEMORY']._loaded_options = None
  _globals['_MEMORY']._serialized_options = b'\232\305\210\036\034\n\032envoy.admin.v2alpha.Memory'
  _globals['_MEMORY']._serialized_start=114
  _globals['_MEMORY']._serialized_end=303
# @@protoc_insertion_point(module_scope)



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Reference implementation for health checking in gRPC Python."""

import collections
import sys
import threading

import grpc
from grpc_health.v1 import health_pb2 as _health_pb2
from grpc_health.v1 import health_pb2_grpc as _health_pb2_grpc

if sys.version_info[0] >= 3 and sys.version_info[1] >= 6:
    # Exposes AsyncHealthServicer as public API.
    from . import _async as aio  # pylint: disable=unused-import

# The service name of the health checking servicer.
SERVICE_NAME = _health_pb2.DESCRIPTOR.services_by_name["Health"].full_name
# The entry of overall health for the entire server.
OVERALL_HEALTH = ""


class _Watcher:
    def __init__(self):
        self._condition = threading.Condition()
        self._responses = collections.deque()
        self._open = True

    def __iter__(self):
        return self

    def _next(self):
        with self._condition:
            while not self._responses and self._open:
                self._condition.wait()
            if self._responses:
                return self._responses.popleft()
            raise StopIteration()

    def next(self):
        return self._next()

    def __next__(self):
        return self._next()

    def add(self, response):
        with self._condition:
            self._responses.append(response)
            self._condition.notify()

    def close(self):
        with self._condition:
            self._open = False
            self._condition.notify()


def _watcher_to_send_response_callback_adapter(watcher):
    def send_response_callback(response):
        if response is None:
            watcher.close()
        else:
            watcher.add(response)

    return send_response_callback


class HealthServicer(_health_pb2_grpc.HealthServicer):
    """Servicer handling RPCs for service statuses."""

    def __init__(
        self, experimental_non_blocking=True, experimental_thread_pool=None
    ):
        self._lock = threading.RLock()
        self._server_status = {"": _health_pb2.HealthCheckResponse.SERVING}
        self._send_response_callbacks = {}
        self.Watch.__func__.experimental_non_blocking = (
            experimental_non_blocking
        )
        self.Watch.__func__.experimental_thread_pool = experimental_thread_pool
        self._gracefully_shutting_down = False

    def _on_close_callback(self, send_response_callback, service):
        def callback():
            with self._lock:
                self._send_response_callbacks[service].remove(
                    send_response_callback
                )
            send_response_callback(None)

        return callback

    def Check(self, request, context):
        with self._lock:
            status = self._server_status.get(request.service)
            if status is None:
                context.set_code(grpc.StatusCode.NOT_FOUND)
                return _health_pb2.HealthCheckResponse()
            return _health_pb2.HealthCheckResponse(status=status)

    # pylint: disable=arguments-differ
    def Watch(self, request, context, send_response_callback=None):
        blocking_watcher = None
        if send_response_callback is None:
            # The server does not support the experimental_non_blocking
            # parameter. For backwards compatibility, return a blocking response
            # generator.
            blocking_watcher = _Watcher()
            send_response_callback = _watcher_to_send_response_callback_adapter(
                blocking_watcher
            )
        service = request.service
        with self._lock:
            status = self._server_status.get(service)
            if status is None:
                status = (
                    _health_pb2.HealthCheckResponse.SERVICE_UNKNOWN
                )  # pylint: disable=no-member
            send_response_callback(
                _health_pb2.HealthCheckResponse(status=status)
            )
            if service not in self._send_response_callbacks:
                self._send_response_callbacks[service] = set()
            self._send_response_callbacks[service].add(send_response_callback)
            context.add_callback(
                self._on_close_callback(send_response_callback, service)
            )
        return blocking_watcher

    def set(self, service, status):
        """Sets the status of a service.

        Args:
          service: string, the name of the service.
          status: HealthCheckResponse.status enum value indicating the status of
            the service
        """
        with self._lock:
            if self._gracefully_shutting_down:
                return
            self._server_status[service] = status
            if service in self._send_response_callbacks:
                for send_response_callback in self._send_response_callbacks[
                    service
                ]:
                    send_response_callback(
                        _health_pb2.HealthCheckResponse(status=status)
                    )

    def enter_graceful_shutdown(self):
        """Permanently sets the status of all services to NOT_SERVING.

        This should be invoked when the server is entering a graceful shutdown
        period. After this method is invoked, future attempts to set the status
        of a service will be ignored.

        This is an EXPERIMENTAL API.
        """
        with self._lock:
            if self._gracefully_shutting_down:
                return
            for service in self._server_status:
                self.set(
                    service, _health_pb2.HealthCheckResponse.NOT_SERVING
                )  # pylint: disable=no-member
            self._gracefully_shutting_down = True



# Copyright 2018 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Implementations of fork support test methods."""

import enum
import json
import logging
import multiprocessing
import os
import queue
import subprocess
import sys
import tempfile
import threading
import time
import traceback

import grpc

from src.proto.grpc.testing import empty_pb2
from src.proto.grpc.testing import messages_pb2
from src.proto.grpc.testing import test_pb2_grpc
from tests.fork import debugger

_LOGGER = logging.getLogger(__name__)
_RPC_TIMEOUT_S = 10
_CHILD_FINISH_TIMEOUT_S = 20
_LIFEBEAT_PERIOD_S = 15


def _channel(args):
    target = "{}:{}".format(args["server_host"], args["server_port"])
    if args["use_tls"]:
        channel_credentials = grpc.ssl_channel_credentials()
        channel = grpc.secure_channel(
            target,
            channel_credentials,
        )
    else:
        channel = grpc.insecure_channel(target)
    return channel


def _validate_payload_type_and_length(response, expected_type, expected_length):
    if response.payload.type is not expected_type:
        raise ValueError(
            "expected payload type %s, got %s"
            % (expected_type, type(response.payload.type))
        )
    elif len(response.payload.body) != expected_length:
        raise ValueError(
            "expected payload body size %d, got %d"
            % (expected_length, len(response.payload.body))
        )


def _async_unary(stub):
    size = 314159
    request = messages_pb2.SimpleRequest(
        response_type=messages_pb2.COMPRESSABLE,
        response_size=size,
        payload=messages_pb2.Payload(body=b"\x00" * 271828),
    )

    response_future = stub.UnaryCall.future(request, timeout=_RPC_TIMEOUT_S)
    response = response_future.result()
    _validate_payload_type_and_length(response, messages_pb2.COMPRESSABLE, size)


def _blocking_unary(stub):
    size = 314159
    request = messages_pb2.SimpleRequest(
        response_type=messages_pb2.COMPRESSABLE,
        response_size=size,
        payload=messages_pb2.Payload(body=b"\x00" * 271828),
    )
    response = stub.UnaryCall(request, timeout=_RPC_TIMEOUT_S)
    _validate_payload_type_and_length(response, messages_pb2.COMPRESSABLE, size)


class _Pipe:
    def __init__(self):
        self._condition = threading.Condition()
        self._values = []
        self._open = True

    def __iter__(self):
        return self

    def __next__(self):
        return self.next()

    def next(self):
        with self._condition:
            while not self._values and self._open:
                self._condition.wait()
            if self._values:
                return self._values.pop(0)
            else:
                raise StopIteration()

    def add(self, value):
        with self._condition:
            self._values.append(value)
            self._condition.notify()

    def close(self):
        with self._condition:
            self._open = False
            self._condition.notify()

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.close()


class _ChildProcess:
    def __init__(self, task, args=None):
        if args is None:
            args = ()
        self._exceptions = multiprocessing.Queue()
        self._stdout_path = tempfile.mkstemp()[1]
        self._stderr_path = tempfile.mkstemp()[1]
        self._child_pid = None
        self._rc = None
        self._args = args
        self._task = task

    def _child_main(self):
        import faulthandler

        faulthandler.enable(all_threads=True)

        try:
            self._task(*self._args)
        except grpc.RpcError as rpc_error:
            traceback.print_exc()
            self._exceptions.put("RpcError: %s" % rpc_error)
        except Exception as e:  # pylint: disable=broad-except
            traceback.print_exc()
            self._exceptions.put(e)
        sys.exit(0)

    def _orchestrate_child_gdb(self):
        cmd = [
            "gdb",
            "-ex",
            "set confirm off",
            "-ex",
            "attach {}".format(os.getpid()),
            "-ex",
            "set follow-fork-mode child",
            "-ex",
            "continue",
            "-ex",
            "bt",
        ]
        streams = tuple(tempfile.TemporaryFile() for _ in range(2))
        sys.stderr.write("Invoking gdb\n")
        sys.stderr.flush()
        process = subprocess.Popen(cmd, stdout=sys.stderr, stderr=sys.stderr)
        time.sleep(5)

    def start(self):
        # NOTE: Try uncommenting the following line if the child is segfaulting.
        # self._orchestrate_child_gdb()
        ret = os.fork()
        if ret == 0:
            self._child_main()
        else:
            self._child_pid = ret
            sys.stderr.write("Child %d started\n" % ret)

    def wait(self, timeout):
        total = 0.0
        wait_interval = 1.0
        next_lifebeat = 0
        while total < timeout:
            ret, termination = os.waitpid(self._child_pid, os.WNOHANG)
            if ret == self._child_pid:
                self._rc = termination
                return True
            time.sleep(wait_interval)
            total += wait_interval
            if total > next_lifebeat:
                sys.stderr.write(
                    "Child %d is still running\n" % self._child_pid
                )
                next_lifebeat = total + _LIFEBEAT_PERIOD_S
        else:
            return False

    def finish(self):
        terminated = self.wait(_CHILD_FINISH_TIMEOUT_S)

        waitstatus = self._rc
        exit_code = os.waitstatus_to_exitcode(waitstatus)
        sys.stderr.write(f"Exit code: {exit_code} ({waitstatus=})\n")

        if not terminated:
            sys.stderr.write("Finishing child %d\n" % self._child_pid)
            debugger.print_backtraces(self._child_pid)
            raise RuntimeError(
                "Child process %d did not terminate" % self._child_pid
            )
        if self._rc != 0:
            raise ValueError(
                f"Child process {self._child_pid} failed"
                f" with {exit_code=} ({waitstatus=})"
            )
        try:
            exception = self._exceptions.get(block=False)
            raise ValueError(
                'Child process %d failed: "%s": "%s"'
                % (self._child_pid, repr(exception), exception)
            )
        except queue.Empty:
            pass


def _async_unary_same_channel(channel):
    def child_target():
        try:
            _async_unary(stub)
        except grpc.RpcError as rpc_error:
            if rpc_error.code() not in (
                grpc.StatusCode.UNAVAILABLE,
                grpc.StatusCode.CANCELLED,
            ):
                raise ValueError("Unexpected status code") from rpc_error
            _async_unary(stub)

    stub = test_pb2_grpc.TestServiceStub(channel)
    _async_unary(stub)
    child_process = _ChildProcess(child_target)
    child_process.start()
    _async_unary(stub)
    child_process.finish()


def _async_unary_new_channel(channel, args):
    def child_target():
        with _channel(args) as child_channel:
            child_stub = test_pb2_grpc.TestServiceStub(child_channel)
            _async_unary(child_stub)

            child_channel.close()

    stub = test_pb2_grpc.TestServiceStub(channel)
    _async_unary(stub)
    child_process = _ChildProcess(child_target)
    child_process.start()
    _async_unary(stub)
    child_process.finish()


def _blocking_unary_same_channel(channel):
    def child_target():
        try:
            _blocking_unary(stub)
        except grpc.RpcError as rpc_error:
            if rpc_error.code() not in (
                grpc.StatusCode.UNAVAILABLE,
                grpc.StatusCode.CANCELLED,
            ):
                raise ValueError("Unexpected status code") from rpc_error
            _async_unary(stub)

    stub = test_pb2_grpc.TestServiceStub(channel)
    _blocking_unary(stub)
    child_process = _ChildProcess(child_target)
    child_process.start()
    child_process.finish()


def _blocking_unary_new_channel(channel, args):
    def child_target():
        with _channel(args) as child_channel:
            child_stub = test_pb2_grpc.TestServiceStub(child_channel)
            _blocking_unary(child_stub)

    stub = test_pb2_grpc.TestServiceStub(channel)
    _blocking_unary(stub)
    child_process = _ChildProcess(child_target)
    child_process.start()
    _blocking_unary(stub)
    child_process.finish()


# Verify that the fork channel registry can handle already closed channels
def _close_channel_before_fork(channel, args):
    def child_target():
        new_channel.close()
        with _channel(args) as child_channel:
            child_stub = test_pb2_grpc.TestServiceStub(child_channel)
            _blocking_unary(child_stub)

    stub = test_pb2_grpc.TestServiceStub(channel)
    _blocking_unary(stub)
    channel.close()

    with _channel(args) as new_channel:
        new_stub = test_pb2_grpc.TestServiceStub(new_channel)
        child_process = _ChildProcess(child_target)
        child_process.start()
        _blocking_unary(new_stub)
        child_process.finish()


def _connectivity_watch(channel, args):
    parent_states = []
    parent_channel_ready_event = threading.Event()

    def child_target():
        child_channel_ready_event = threading.Event()

        def child_connectivity_callback(state):
            if state is grpc.ChannelConnectivity.READY:
                child_channel_ready_event.set()

        with _channel(args) as child_channel:
            child_stub = test_pb2_grpc.TestServiceStub(child_channel)
            child_channel.subscribe(child_connectivity_callback)
            _async_unary(child_stub)
            _async_unary(stub)
            if not child_channel_ready_event.wait(timeout=_RPC_TIMEOUT_S):
                raise ValueError("Child channel did not move to READY")
            if not parent_channel_ready_event.wait(timeout=_RPC_TIMEOUT_S):
                raise ValueError("Parent channel did not move to READY")
            child_channel.unsubscribe(child_connectivity_callback)
            channel.unsubscribe(parent_connectivity_callback)

    def parent_connectivity_callback(state):
        parent_states.append(state)
        if state is grpc.ChannelConnectivity.READY:
            parent_channel_ready_event.set()

    channel.subscribe(parent_connectivity_callback)
    stub = test_pb2_grpc.TestServiceStub(channel)
    child_process = _ChildProcess(child_target)
    child_process.start()
    _async_unary(stub)
    if not parent_channel_ready_event.wait(timeout=_RPC_TIMEOUT_S):
        raise ValueError("Channel did not move to READY")
    channel.unsubscribe(parent_connectivity_callback)
    child_process.finish()


def _ping_pong_with_child_processes_after_first_response(
    channel, args, child_target, run_after_close=True
):
    request_response_sizes = (
        31415,
        9,
        2653,
        58979,
    )
    request_payload_sizes = (
        27182,
        8,
        1828,
        45904,
    )
    stub = test_pb2_grpc.TestServiceStub(channel)
    pipe = _Pipe()
    parent_bidi_call = stub.FullDuplexCall(pipe)
    child_processes = []
    first_message_received = False
    for response_size, payload_size in zip(
        request_response_sizes, request_payload_sizes
    ):
        request = messages_pb2.StreamingOutputCallRequest(
            response_type=messages_pb2.COMPRESSABLE,
            response_parameters=(
                messages_pb2.ResponseParameters(size=response_size),
            ),
            payload=messages_pb2.Payload(body=b"\x00" * payload_size),
        )
        pipe.add(request)
        if first_message_received:
            child_process = _ChildProcess(
                child_target, (parent_bidi_call, channel, args)
            )
            child_process.start()
            child_processes.append(child_process)
        response = next(parent_bidi_call)
        first_message_received = True
        child_process = _ChildProcess(
            child_target, (parent_bidi_call, channel, args)
        )
        child_process.start()
        child_processes.append(child_process)
        _validate_payload_type_and_length(
            response, messages_pb2.COMPRESSABLE, response_size
        )
    pipe.close()
    if run_after_close:
        child_process = _ChildProcess(
            child_target, (parent_bidi_call, channel, args)
        )
        child_process.start()
        child_processes.append(child_process)
    for child_process in child_processes:
        child_process.finish()


def _in_progress_bidi_continue_call(channel):
    def child_target(parent_bidi_call, parent_channel, args):
        stub = test_pb2_grpc.TestServiceStub(parent_channel)
        try:
            _async_unary(stub)
        except grpc.RpcError as rpc_error:
            if rpc_error.code() not in (
                grpc.StatusCode.UNAVAILABLE,
                grpc.StatusCode.CANCELLED,
            ):
                raise ValueError("Unexpected status code") from rpc_error
            _async_unary(stub)
        inherited_code = parent_bidi_call.code()
        if inherited_code != grpc.StatusCode.CANCELLED:
            raise ValueError(
                "Expected inherited code CANCELLED, "
                f"got {inherited_code} (details: {parent_bidi_call.details()})"
            )

    # Don't run child_target after closing the parent call, as the call may have
    # received a status from the  server before fork occurs.
    _ping_pong_with_child_processes_after_first_response(
        channel, None, child_target, run_after_close=False
    )


def _in_progress_bidi_same_channel_async_call(channel):
    def child_target(parent_bidi_call, parent_channel, args):
        stub = test_pb2_grpc.TestServiceStub(parent_channel)
        try:
            _async_unary(stub)
        except grpc.RpcError as rpc_error:
            if rpc_error.code() not in (
                grpc.StatusCode.UNAVAILABLE,
                grpc.StatusCode.CANCELLED,
            ):
                raise ValueError("Unexpected status code") from rpc_error
            _async_unary(stub)

    _ping_pong_with_child_processes_after_first_response(
        channel, None, child_target
    )


def _in_progress_bidi_same_channel_blocking_call(channel):
    def child_target(parent_bidi_call, parent_channel, args):
        stub = test_pb2_grpc.TestServiceStub(parent_channel)
        try:
            _blocking_unary(stub)
        except grpc.RpcError as rpc_error:
            if rpc_error.code() not in (
                grpc.StatusCode.UNAVAILABLE,
                grpc.StatusCode.CANCELLED,
            ):
                raise ValueError("Unexpected status code") from rpc_error
            _async_unary(stub)

    _ping_pong_with_child_processes_after_first_response(
        channel, None, child_target
    )


def _in_progress_bidi_new_channel_async_call(channel, args):
    def child_target(parent_bidi_call, parent_channel, args):
        with _channel(args) as channel:
            stub = test_pb2_grpc.TestServiceStub(channel)
            _async_unary(stub)

    _ping_pong_with_child_processes_after_first_response(
        channel, args, child_target
    )


def _in_progress_bidi_new_channel_blocking_call(channel, args):
    def child_target(parent_bidi_call, parent_channel, args):
        with _channel(args) as channel:
            stub = test_pb2_grpc.TestServiceStub(channel)
            _blocking_unary(stub)

    _ping_pong_with_child_processes_after_first_response(
        channel, args, child_target
    )


@enum.unique
class TestCase(enum.Enum):
    CONNECTIVITY_WATCH = "connectivity_watch"
    CLOSE_CHANNEL_BEFORE_FORK = "close_channel_before_fork"
    ASYNC_UNARY_SAME_CHANNEL = "async_unary_same_channel"
    ASYNC_UNARY_NEW_CHANNEL = "async_unary_new_channel"
    BLOCKING_UNARY_SAME_CHANNEL = "blocking_unary_same_channel"
    BLOCKING_UNARY_NEW_CHANNEL = "blocking_unary_new_channel"
    IN_PROGRESS_BIDI_CONTINUE_CALL = "in_progress_bidi_continue_call"
    IN_PROGRESS_BIDI_SAME_CHANNEL_ASYNC_CALL = (
        "in_progress_bidi_same_channel_async_call"
    )
    IN_PROGRESS_BIDI_SAME_CHANNEL_BLOCKING_CALL = (
        "in_progress_bidi_same_channel_blocking_call"
    )
    IN_PROGRESS_BIDI_NEW_CHANNEL_ASYNC_CALL = (
        "in_progress_bidi_new_channel_async_call"
    )
    IN_PROGRESS_BIDI_NEW_CHANNEL_BLOCKING_CALL = (
        "in_progress_bidi_new_channel_blocking_call"
    )

    def run_test(self, args):
        _LOGGER.info("Running %s", self)
        channel = _channel(args)
        if self is TestCase.ASYNC_UNARY_SAME_CHANNEL:
            _async_unary_same_channel(channel)
        elif self is TestCase.ASYNC_UNARY_NEW_CHANNEL:
            _async_unary_new_channel(channel, args)
        elif self is TestCase.BLOCKING_UNARY_SAME_CHANNEL:
            _blocking_unary_same_channel(channel)
        elif self is TestCase.BLOCKING_UNARY_NEW_CHANNEL:
            _blocking_unary_new_channel(channel, args)
        elif self is TestCase.CLOSE_CHANNEL_BEFORE_FORK:
            _close_channel_before_fork(channel, args)
        elif self is TestCase.CONNECTIVITY_WATCH:
            _connectivity_watch(channel, args)
        elif self is TestCase.IN_PROGRESS_BIDI_CONTINUE_CALL:
            _in_progress_bidi_continue_call(channel)
        elif self is TestCase.IN_PROGRESS_BIDI_SAME_CHANNEL_ASYNC_CALL:
            _in_progress_bidi_same_channel_async_call(channel)
        elif self is TestCase.IN_PROGRESS_BIDI_SAME_CHANNEL_BLOCKING_CALL:
            _in_progress_bidi_same_channel_blocking_call(channel)
        elif self is TestCase.IN_PROGRESS_BIDI_NEW_CHANNEL_ASYNC_CALL:
            _in_progress_bidi_new_channel_async_call(channel, args)
        elif self is TestCase.IN_PROGRESS_BIDI_NEW_CHANNEL_BLOCKING_CALL:
            _in_progress_bidi_new_channel_blocking_call(channel, args)
        else:
            raise NotImplementedError(
                'Test case "%s" not implemented!' % self.name
            )
        channel.close()


# Useful if needing to find a block of code from an address in an SO.
def dump_object_map():
    with open("/proc/self/maps", "r") as f:
        sys.stderr.write("=============== /proc/self/maps ===============\n")
        sys.stderr.write(f.read())
        sys.stderr.write("\n")
        sys.stderr.flush()


def setup_logger():
    logging.basicConfig(
        level=logging.INFO,
        style="{",
        format=(
            "{levelname[0]}{asctime}.{msecs:03.0f} {process}/{thread} "
            "{filename}:{lineno}] {message}"
        ),
        datefmt="%m%d %H:%M:%S",
    )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: google/rpc/context/audit_context.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'google/rpc/context/audit_context.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&google/rpc/context/audit_context.proto\x12\x12google.rpc.context\x1a\x1cgoogle/protobuf/struct.proto\"\xc7\x01\n\x0c\x41uditContext\x12\x11\n\taudit_log\x18\x01 \x01(\x0c\x12\x31\n\x10scrubbed_request\x18\x02 \x01(\x0b\x32\x17.google.protobuf.Struct\x12\x32\n\x11scrubbed_response\x18\x03 \x01(\x0b\x32\x17.google.protobuf.Struct\x12$\n\x1cscrubbed_response_item_count\x18\x04 \x01(\x05\x12\x17\n\x0ftarget_resource\x18\x05 \x01(\tBh\n\x16\x63om.google.rpc.contextB\x11\x41uditContextProtoP\x01Z9google.golang.org/genproto/googleapis/rpc/context;contextb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'google.rpc.context.audit_context_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\026com.google.rpc.contextB\021AuditContextProtoP\001Z9google.golang.org/genproto/googleapis/rpc/context;context'
  _globals['_AUDITCONTEXT']._serialized_start=93
  _globals['_AUDITCONTEXT']._serialized_end=292
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python3
# Copyright 2021 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# This script finds and eliminates redundant namespace qualifiers.
#
# It is optimized to run in parallel using all available CPU cores.
#
# USAGE:
#
# 1. To check the entire repository (the default CI behavior):
#    ./check_redundant_namespace_qualifiers.py
#
# 2. To check only specific files (e.g., files changed in your git branch):
#    git diff --name-only main... | xargs ./check_redundant_namespace_qualifiers.py
#

import collections
import multiprocessing
import os
import re
import sys

IGNORED_FILES = [
    # note: the grpc_core::Server redundant namespace qualification is required
    # for older gcc versions.
    "src/core/ext/transport/chttp2/server/chttp2_server.h",
    "src/core/server/server.h",
    # generated code adds a necessary grpc_core:: for a logging macro which can
    # be used anywhere.
    "src/core/lib/debug/trace_impl.h",
]


def find_closing_mustache(contents, initial_depth):
    """Find the closing mustache for a given number of open mustaches."""
    depth = initial_depth
    start_len = len(contents)
    while contents:
        # Skip over raw strings.
        if contents.startswith('R"'):
            contents = contents[2:]
            offset = contents.find("(")
            if offset == -1:
                return None  # Malformed raw string
            prefix = contents[:offset]
            contents = contents[offset:]
            end_marker = f'){prefix}"'
            offset = contents.find(end_marker)
            if offset == -1:
                return None  # Malformed raw string
            contents = contents[offset + len(end_marker) :]
        # Skip over strings.
        elif contents[0] == '"':
            contents = contents[1:]
            while len(contents) > 0 and contents[0] != '"':
                if contents.startswith(("\\\\", '\\"')):
                    contents = contents[2:]
                else:
                    contents = contents[1:]
            if len(contents) > 0:
                contents = contents[1:]
        # And characters that might confuse us.
        elif (
            contents.startswith("'{'")
            or contents.startswith("'\"'")
            or contents.startswith("'}'")
        ):
            contents = contents[3:]
        # Skip over comments.
        elif contents.startswith("//"):
            offset = contents.find("\n")
            if offset == -1:
                break
            contents = contents[offset:]
        elif contents.startswith("/*"):
            offset = contents.find("*/")
            if offset == -1:
                break
            contents = contents[offset + 2 :]
        # Count up or down if we see a mustache.
        elif contents[0] == "{":
            contents = contents[1:]
            depth += 1
        elif contents[0] == "}":
            contents = contents[1:]
            depth -= 1
            if depth == 0:
                return start_len - len(contents)
        # Skip over everything else.
        else:
            contents = contents[1:]
    return None


def is_a_define_statement(match, body):
    """See if the matching line begins with #define"""
    # This does not yet help with multi-line defines
    m = re.search(
        r"^#define.*{}$".format(re.escape(match.group(0))),
        body[: match.end()],
        re.MULTILINE,
    )
    return m is not None


def update_file(contents, namespaces):
    """Scan the contents of a file, and for top-level namespaces in namespaces remove redundant usages."""
    parts = []
    while contents:
        m = re.search(r"namespace\s+([a-zA-Z0-9_]*)\s*{", contents)
        if not m:
            parts.append(contents)
            break
        parts.append(contents[: m.end()])
        contents = contents[m.end() :]
        end = find_closing_mustache(contents, 1)
        if end is None:
            # Could not find a match, return original content to be safe
            parts.append(contents)
            return "".join(parts)

        body = contents[:end]
        namespace = m.group(1)
        if namespace in namespaces:
            body_parts = []
            while body:
                # Find instances of 'namespace::'
                m = re.search(r"\b" + namespace + r"::\b", body)
                if not m:
                    break
                # Ignore instances of '::namespace::' -- these are usually meant to be there.
                if m.start() >= 2 and body[m.start() - 2 :].startswith("::"):
                    body_parts.append(body[: m.end()])
                # Ignore #defines, since they may be used anywhere
                elif is_a_define_statement(m, body):
                    body_parts.append(body[: m.end()])
                else:
                    body_parts.append(body[: m.start()])
                body = body[m.end() :]
            body_parts.append(body)
            parts.append("".join(body_parts))
        else:
            parts.append(body)

        contents = contents[end:]
    return "".join(parts)


def run_self_check():
    """Run a self-check before doing anything."""
    _TEST = """
    namespace bar {
        namespace baz {
        }
    }
    namespace foo {}
    namespace foo {
        foo::a;
        ::foo::a;
    }
    {R"foo({
    foo::a;
    }foo)"}
    """
    _TEST_EXPECTED = """
    namespace bar {
        namespace baz {
        }
    }
    namespace foo {}
    namespace foo {
        a;
        ::foo::a;
    }
    {R"foo({
    foo::a;
    }foo)"}
    """
    output = update_file(_TEST, ["foo"])
    if output != _TEST_EXPECTED:
        import difflib

        print("FAILED: self check")
        print(
            "\n".join(
                difflib.ndiff(
                    _TEST_EXPECTED.splitlines(1), output.splitlines(1)
                )
            )
        )
        sys.exit(1)


# Define a configuration tuple for clarity.
Config = collections.namedtuple("Config", ["dirs", "namespaces"])
_CONFIGURATION = (Config(("src/core", "test/core"), ("grpc_core",)),)


def process_file(path_and_config):
    """
    Processes a single file. Reads, updates, and writes back if changed.
    Returns the file path if changed, otherwise None.
    This function is designed to be called from a multiprocessing Pool.
    """
    path, config = path_and_config
    if any(ignored in path for ignored in IGNORED_FILES):
        return None
    try:
        with open(path) as f:
            contents = f.read()
    except IOError:
        return None  # Skip files that can't be opened

    try:
        updated = update_file(contents, config.namespaces)
        if updated != contents:
            with open(path, "w") as f:
                f.write(updated)
            return path
    except Exception as e:
        print(f"ERROR processing {path}: {e}", file=sys.stderr)
    return None


def main():
    """Main loop."""
    run_self_check()

    # If file paths are passed as arguments, use them.
    # Otherwise, walk the directories defined in the configuration.
    args = sys.argv[1:]
    if args:
        # For scoped runs, we must determine the config for each file.
        files_to_check = []
        for path in args:
            for config in _CONFIGURATION:
                if any(path.startswith(d) for d in config.dirs):
                    files_to_check.append((path, config))
                    break
    else:
        # Prepare a list of all files to be checked.
        files_to_check = []
        for config in _CONFIGURATION:
            for directory in config.dirs:
                for root, _, files in os.walk(directory):
                    for file in files:
                        if file.endswith((".cc", ".h")):
                            path = os.path.join(root, file)
                            files_to_check.append((path, config))

    if not files_to_check:
        return

    print(f"Checking {len(files_to_check)} files for redundant namespaces...")

    # Use a multiprocessing Pool to process files in parallel.
    # The number of processes will default to the number of CPU cores.
    with multiprocessing.Pool() as pool:
        # imap_unordered is used for efficiency, processing results as they complete.
        results = pool.imap_unordered(process_file, files_to_check)
        changed = [result for result in results if result is not None]

    if changed:
        print("The following files were changed:")
        for path in sorted(changed):
            print(f"  {path}")
        sys.exit(1)
    else:
        print("No files needed changes.")


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: contrib/envoy/extensions/filters/http/sxg/v3alpha/sxg.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'contrib/envoy/extensions/filters/http/sxg/v3alpha/sxg.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.extensions.transport_sockets.tls.v3 import secret_pb2 as envoy_dot_extensions_dot_transport__sockets_dot_tls_dot_v3_dot_secret__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n;contrib/envoy/extensions/filters/http/sxg/v3alpha/sxg.proto\x12)envoy.extensions.filters.http.sxg.v3alpha\x1a\x36\x65nvoy/extensions/transport_sockets/tls/v3/secret.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xc5\x03\n\x03SXG\x12O\n\x0b\x63\x65rtificate\x18\x01 \x01(\x0b\x32:.envoy.extensions.transport_sockets.tls.v3.SdsSecretConfig\x12O\n\x0bprivate_key\x18\x02 \x01(\x0b\x32:.envoy.extensions.transport_sockets.tls.v3.SdsSecretConfig\x12+\n\x08\x64uration\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x16\n\x0emi_record_size\x18\x04 \x01(\x04\x12\x1c\n\x08\x63\x62or_url\x18\x05 \x01(\tB\n\xfa\x42\x07r\x05\x10\x01:\x01/\x12 \n\x0cvalidity_url\x18\x06 \x01(\tB\n\xfa\x42\x07r\x05\x10\x01:\x01/\x12\x34\n\x1c\x63lient_can_accept_sxg_header\x18\x07 \x01(\tB\x0e\xfa\x42\x0br\t\xc0\x01\x01\xc8\x01\x00\xd0\x01\x01\x12\x30\n\x18should_encode_sxg_header\x18\x08 \x01(\tB\x0e\xfa\x42\x0br\t\xc0\x01\x01\xc8\x01\x00\xd0\x01\x01\x12/\n\x15header_prefix_filters\x18\t \x03(\tB\x10\xfa\x42\r\x92\x01\n\"\x08r\x06\xc0\x01\x01\xc8\x01\x00\x42\xa9\x01\n7io.envoyproxy.envoy.extensions.filters.http.sxg.v3alphaB\x08SxgProtoP\x01ZXgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/http/sxg/v3alpha\xba\x80\xc8\xd1\x06\x04\x08\x01\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'contrib.envoy.extensions.filters.http.sxg.v3alpha.sxg_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n7io.envoyproxy.envoy.extensions.filters.http.sxg.v3alphaB\010SxgProtoP\001ZXgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/http/sxg/v3alpha\272\200\310\321\006\004\010\001\020\002'
  _globals['_SXG'].fields_by_name['cbor_url']._loaded_options = None
  _globals['_SXG'].fields_by_name['cbor_url']._serialized_options = b'\372B\007r\005\020\001:\001/'
  _globals['_SXG'].fields_by_name['validity_url']._loaded_options = None
  _globals['_SXG'].fields_by_name['validity_url']._serialized_options = b'\372B\007r\005\020\001:\001/'
  _globals['_SXG'].fields_by_name['client_can_accept_sxg_header']._loaded_options = None
  _globals['_SXG'].fields_by_name['client_can_accept_sxg_header']._serialized_options = b'\372B\013r\t\300\001\001\310\001\000\320\001\001'
  _globals['_SXG'].fields_by_name['should_encode_sxg_header']._loaded_options = None
  _globals['_SXG'].fields_by_name['should_encode_sxg_header']._serialized_options = b'\372B\013r\t\300\001\001\310\001\000\320\001\001'
  _globals['_SXG'].fields_by_name['header_prefix_filters']._loaded_options = None
  _globals['_SXG'].fields_by_name['header_prefix_filters']._serialized_options = b'\372B\r\222\001\n\"\010r\006\300\001\001\310\001\000'
  _globals['_SXG']._serialized_start=251
  _globals['_SXG']._serialized_end=704
# @@protoc_insertion_point(module_scope)



# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Testing the done callbacks mechanism."""

import asyncio
import gc
import logging
import platform
import time
import unittest

import grpc
from grpc.experimental import aio

from src.proto.grpc.testing import messages_pb2
from src.proto.grpc.testing import test_pb2_grpc
from tests.unit.framework.common import get_socket
from tests.unit.framework.common import test_constants
from tests_aio.unit import _common
from tests_aio.unit._test_base import AioTestBase
from tests_aio.unit._test_server import start_test_server

_NUM_STREAM_RESPONSES = 5
_REQUEST_PAYLOAD_SIZE = 7
_RESPONSE_PAYLOAD_SIZE = 42


async def _perform_unary_unary(stub, wait_for_ready):
    await stub.UnaryCall(
        messages_pb2.SimpleRequest(),
        timeout=test_constants.LONG_TIMEOUT,
        wait_for_ready=wait_for_ready,
    )


async def _perform_unary_stream(stub, wait_for_ready):
    request = messages_pb2.StreamingOutputCallRequest()
    for _ in range(_NUM_STREAM_RESPONSES):
        request.response_parameters.append(
            messages_pb2.ResponseParameters(size=_RESPONSE_PAYLOAD_SIZE)
        )

    call = stub.StreamingOutputCall(
        request,
        timeout=test_constants.LONG_TIMEOUT,
        wait_for_ready=wait_for_ready,
    )

    for _ in range(_NUM_STREAM_RESPONSES):
        await call.read()
    assert await call.code() == grpc.StatusCode.OK


async def _perform_stream_unary(stub, wait_for_ready):
    payload = messages_pb2.Payload(body=b"\0" * _REQUEST_PAYLOAD_SIZE)
    request = messages_pb2.StreamingInputCallRequest(payload=payload)

    async def gen():
        for _ in range(_NUM_STREAM_RESPONSES):
            yield request

    await stub.StreamingInputCall(
        gen(),
        timeout=test_constants.LONG_TIMEOUT,
        wait_for_ready=wait_for_ready,
    )


async def _perform_stream_stream(stub, wait_for_ready):
    call = stub.FullDuplexCall(
        timeout=test_constants.LONG_TIMEOUT, wait_for_ready=wait_for_ready
    )

    request = messages_pb2.StreamingOutputCallRequest()
    request.response_parameters.append(
        messages_pb2.ResponseParameters(size=_RESPONSE_PAYLOAD_SIZE)
    )

    for _ in range(_NUM_STREAM_RESPONSES):
        await call.write(request)
        response = await call.read()
        assert _RESPONSE_PAYLOAD_SIZE == len(response.payload.body)

    await call.done_writing()
    assert await call.code() == grpc.StatusCode.OK


_RPC_ACTIONS = (
    _perform_unary_unary,
    _perform_unary_stream,
    _perform_stream_unary,
    _perform_stream_stream,
)


class TestWaitForReady(AioTestBase):
    async def setUp(self):
        address, self._port, self._socket = get_socket(listen=False)
        self._channel = aio.insecure_channel(f"{address}:{self._port}")
        self._stub = test_pb2_grpc.TestServiceStub(self._channel)
        self._socket.close()

    async def tearDown(self):
        await self._channel.close()

    async def _connection_fails_fast(self, wait_for_ready):
        for action in _RPC_ACTIONS:
            with self.subTest(name=action):
                with self.assertRaises(aio.AioRpcError) as exception_context:
                    await action(self._stub, wait_for_ready)
                rpc_error = exception_context.exception
                self.assertEqual(grpc.StatusCode.UNAVAILABLE, rpc_error.code())

    async def test_call_wait_for_ready_default(self):
        """RPC should fail immediately after connection failed."""
        await self._connection_fails_fast(None)

    async def test_call_wait_for_ready_disabled(self):
        """RPC should fail immediately after connection failed."""
        await self._connection_fails_fast(False)

    @unittest.skipIf(
        platform.system() == "Windows",
        "https://github.com/grpc/grpc/pull/26729",
    )
    async def test_call_wait_for_ready_enabled(self):
        """RPC will wait until the connection is ready."""
        for action in _RPC_ACTIONS:
            with self.subTest(name=action.__name__):
                # Starts the RPC
                action_task = self.loop.create_task(action(self._stub, True))

                # Wait for TRANSIENT_FAILURE, and RPC is not aborting
                await _common.block_until_certain_state(
                    self._channel, grpc.ChannelConnectivity.TRANSIENT_FAILURE
                )

                server = None

                try:
                    # Start the server
                    _, server = await start_test_server(port=self._port)

                    # The RPC should recover itself
                    await action_task
                finally:
                    if server is not None:
                        await server.stop(None)


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main(verbosity=2)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: google/logging/v2/logging_metrics.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'google/logging/v2/logging_metrics.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.api import client_pb2 as google_dot_api_dot_client__pb2
from google.api import distribution_pb2 as google_dot_api_dot_distribution__pb2
from google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from google.api import metric_pb2 as google_dot_api_dot_metric__pb2
from google.api import resource_pb2 as google_dot_api_dot_resource__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'google/logging/v2/logging_metrics.proto\x12\x11google.logging.v2\x1a\x1cgoogle/api/annotations.proto\x1a\x17google/api/client.proto\x1a\x1dgoogle/api/distribution.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x17google/api/metric.proto\x1a\x19google/api/resource.proto\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xbd\x05\n\tLogMetric\x12\x11\n\x04name\x18\x01 \x01(\tB\x03\xe0\x41\x02\x12\x18\n\x0b\x64\x65scription\x18\x02 \x01(\tB\x03\xe0\x41\x01\x12\x13\n\x06\x66ilter\x18\x03 \x01(\tB\x03\xe0\x41\x02\x12\x18\n\x0b\x62ucket_name\x18\r \x01(\tB\x03\xe0\x41\x01\x12\x15\n\x08\x64isabled\x18\x0c \x01(\x08\x42\x03\xe0\x41\x01\x12<\n\x11metric_descriptor\x18\x05 \x01(\x0b\x32\x1c.google.api.MetricDescriptorB\x03\xe0\x41\x01\x12\x1c\n\x0fvalue_extractor\x18\x06 \x01(\tB\x03\xe0\x41\x01\x12P\n\x10label_extractors\x18\x07 \x03(\x0b\x32\x31.google.logging.v2.LogMetric.LabelExtractorsEntryB\x03\xe0\x41\x01\x12\x43\n\x0e\x62ucket_options\x18\x08 \x01(\x0b\x32&.google.api.Distribution.BucketOptionsB\x03\xe0\x41\x01\x12\x34\n\x0b\x63reate_time\x18\t \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x03\xe0\x41\x03\x12\x34\n\x0bupdate_time\x18\n \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x03\xe0\x41\x03\x12<\n\x07version\x18\x04 \x01(\x0e\x32\'.google.logging.v2.LogMetric.ApiVersionB\x02\x18\x01\x1a\x36\n\x14LabelExtractorsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x1c\n\nApiVersion\x12\x06\n\x02V2\x10\x00\x12\x06\n\x02V1\x10\x01:J\xea\x41G\n logging.googleapis.com/LogMetric\x12#projects/{project}/metrics/{metric}\"\x8d\x01\n\x15ListLogMetricsRequest\x12\x43\n\x06parent\x18\x01 \x01(\tB3\xe0\x41\x02\xfa\x41-\n+cloudresourcemanager.googleapis.com/Project\x12\x17\n\npage_token\x18\x02 \x01(\tB\x03\xe0\x41\x01\x12\x16\n\tpage_size\x18\x03 \x01(\x05\x42\x03\xe0\x41\x01\"`\n\x16ListLogMetricsResponse\x12-\n\x07metrics\x18\x01 \x03(\x0b\x32\x1c.google.logging.v2.LogMetric\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t\"T\n\x13GetLogMetricRequest\x12=\n\x0bmetric_name\x18\x01 \x01(\tB(\xe0\x41\x02\xfa\x41\"\n logging.googleapis.com/LogMetric\"\x85\x01\n\x16\x43reateLogMetricRequest\x12\x38\n\x06parent\x18\x01 \x01(\tB(\xe0\x41\x02\xfa\x41\"\x12 logging.googleapis.com/LogMetric\x12\x31\n\x06metric\x18\x02 \x01(\x0b\x32\x1c.google.logging.v2.LogMetricB\x03\xe0\x41\x02\"\x8a\x01\n\x16UpdateLogMetricRequest\x12=\n\x0bmetric_name\x18\x01 \x01(\tB(\xe0\x41\x02\xfa\x41\"\n logging.googleapis.com/LogMetric\x12\x31\n\x06metric\x18\x02 \x01(\x0b\x32\x1c.google.logging.v2.LogMetricB\x03\xe0\x41\x02\"W\n\x16\x44\x65leteLogMetricRequest\x12=\n\x0bmetric_name\x18\x01 \x01(\tB(\xe0\x41\x02\xfa\x41\"\n logging.googleapis.com/LogMetric2\xae\x08\n\x10MetricsServiceV2\x12\x97\x01\n\x0eListLogMetrics\x12(.google.logging.v2.ListLogMetricsRequest\x1a).google.logging.v2.ListLogMetricsResponse\"0\xda\x41\x06parent\x82\xd3\xe4\x93\x02!\x12\x1f/v2/{parent=projects/*}/metrics\x12\x92\x01\n\x0cGetLogMetric\x12&.google.logging.v2.GetLogMetricRequest\x1a\x1c.google.logging.v2.LogMetric\"<\xda\x41\x0bmetric_name\x82\xd3\xe4\x93\x02(\x12&/v2/{metric_name=projects/*/metrics/*}\x12\x9b\x01\n\x0f\x43reateLogMetric\x12).google.logging.v2.CreateLogMetricRequest\x1a\x1c.google.logging.v2.LogMetric\"?\xda\x41\rparent,metric\x82\xd3\xe4\x93\x02)\"\x1f/v2/{parent=projects/*}/metrics:\x06metric\x12\xa7\x01\n\x0fUpdateLogMetric\x12).google.logging.v2.UpdateLogMetricRequest\x1a\x1c.google.logging.v2.LogMetric\"K\xda\x41\x12metric_name,metric\x82\xd3\xe4\x93\x02\x30\x1a&/v2/{metric_name=projects/*/metrics/*}:\x06metric\x12\x92\x01\n\x0f\x44\x65leteLogMetric\x12).google.logging.v2.DeleteLogMetricRequest\x1a\x16.google.protobuf.Empty\"<\xda\x41\x0bmetric_name\x82\xd3\xe4\x93\x02(*&/v2/{metric_name=projects/*/metrics/*}\x1a\x8d\x02\xca\x41\x16logging.googleapis.com\xd2\x41\xf0\x01https://www.googleapis.com/auth/cloud-platform,https://www.googleapis.com/auth/cloud-platform.read-only,https://www.googleapis.com/auth/logging.admin,https://www.googleapis.com/auth/logging.read,https://www.googleapis.com/auth/logging.writeB\xb9\x01\n\x15\x63om.google.logging.v2B\x13LoggingMetricsProtoP\x01Z5cloud.google.com/go/logging/apiv2/loggingpb;loggingpb\xf8\x01\x01\xaa\x02\x17Google.Cloud.Logging.V2\xca\x02\x17Google\\Cloud\\Logging\\V2\xea\x02\x1aGoogle::Cloud::Logging::V2b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'google.logging.v2.logging_metrics_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\025com.google.logging.v2B\023LoggingMetricsProtoP\001Z5cloud.google.com/go/logging/apiv2/loggingpb;loggingpb\370\001\001\252\002\027Google.Cloud.Logging.V2\312\002\027Google\\Cloud\\Logging\\V2\352\002\032Google::Cloud::Logging::V2'
  _globals['_LOGMETRIC_LABELEXTRACTORSENTRY']._loaded_options = None
  _globals['_LOGMETRIC_LABELEXTRACTORSENTRY']._serialized_options = b'8\001'
  _globals['_LOGMETRIC'].fields_by_name['name']._loaded_options = None
  _globals['_LOGMETRIC'].fields_by_name['name']._serialized_options = b'\340A\002'
  _globals['_LOGMETRIC'].fields_by_name['description']._loaded_options = None
  _globals['_LOGMETRIC'].fields_by_name['description']._serialized_options = b'\340A\001'
  _globals['_LOGMETRIC'].fields_by_name['filter']._loaded_options = None
  _globals['_LOGMETRIC'].fields_by_name['filter']._serialized_options = b'\340A\002'
  _globals['_LOGMETRIC'].fields_by_name['bucket_name']._loaded_options = None
  _globals['_LOGMETRIC'].fields_by_name['bucket_name']._serialized_options = b'\340A\001'
  _globals['_LOGMETRIC'].fields_by_name['disabled']._loaded_options = None
  _globals['_LOGMETRIC'].fields_by_name['disabled']._serialized_options = b'\340A\001'
  _globals['_LOGMETRIC'].fields_by_name['metric_descriptor']._loaded_options = None
  _globals['_LOGMETRIC'].fields_by_name['metric_descriptor']._serialized_options = b'\340A\001'
  _globals['_LOGMETRIC'].fields_by_name['value_extractor']._loaded_options = None
  _globals['_LOGMETRIC'].fields_by_name['value_extractor']._serialized_options = b'\340A\001'
  _globals['_LOGMETRIC'].fields_by_name['label_extractors']._loaded_options = None
  _globals['_LOGMETRIC'].fields_by_name['label_extractors']._serialized_options = b'\340A\001'
  _globals['_LOGMETRIC'].fields_by_name['bucket_options']._loaded_options = None
  _globals['_LOGMETRIC'].fields_by_name['bucket_options']._serialized_options = b'\340A\001'
  _globals['_LOGMETRIC'].fields_by_name['create_time']._loaded_options = None
  _globals['_LOGMETRIC'].fields_by_name['create_time']._serialized_options = b'\340A\003'
  _globals['_LOGMETRIC'].fields_by_name['update_time']._loaded_options = None
  _globals['_LOGMETRIC'].fields_by_name['update_time']._serialized_options = b'\340A\003'
  _globals['_LOGMETRIC'].fields_by_name['version']._loaded_options = None
  _globals['_LOGMETRIC'].fields_by_name['version']._serialized_options = b'\030\001'
  _globals['_LOGMETRIC']._loaded_options = None
  _globals['_LOGMETRIC']._serialized_options = b'\352AG\n logging.googleapis.com/LogMetric\022#projects/{project}/metrics/{metric}'
  _globals['_LISTLOGMETRICSREQUEST'].fields_by_name['parent']._loaded_options = None
  _globals['_LISTLOGMETRICSREQUEST'].fields_by_name['parent']._serialized_options = b'\340A\002\372A-\n+cloudresourcemanager.googleapis.com/Project'
  _globals['_LISTLOGMETRICSREQUEST'].fields_by_name['page_token']._loaded_options = None
  _globals['_LISTLOGMETRICSREQUEST'].fields_by_name['page_token']._serialized_options = b'\340A\001'
  _globals['_LISTLOGMETRICSREQUEST'].fields_by_name['page_size']._loaded_options = None
  _globals['_LISTLOGMETRICSREQUEST'].fields_by_name['page_size']._serialized_options = b'\340A\001'
  _globals['_GETLOGMETRICREQUEST'].fields_by_name['metric_name']._loaded_options = None
  _globals['_GETLOGMETRICREQUEST'].fields_by_name['metric_name']._serialized_options = b'\340A\002\372A\"\n logging.googleapis.com/LogMetric'
  _globals['_CREATELOGMETRICREQUEST'].fields_by_name['parent']._loaded_options = None
  _globals['_CREATELOGMETRICREQUEST'].fields_by_name['parent']._serialized_options = b'\340A\002\372A\"\022 logging.googleapis.com/LogMetric'
  _globals['_CREATELOGMETRICREQUEST'].fields_by_name['metric']._loaded_options = None
  _globals['_CREATELOGMETRICREQUEST'].fields_by_name['metric']._serialized_options = b'\340A\002'
  _globals['_UPDATELOGMETRICREQUEST'].fields_by_name['metric_name']._loaded_options = None
  _globals['_UPDATELOGMETRICREQUEST'].fields_by_name['metric_name']._serialized_options = b'\340A\002\372A\"\n logging.googleapis.com/LogMetric'
  _globals['_UPDATELOGMETRICREQUEST'].fields_by_name['metric']._loaded_options = None
  _globals['_UPDATELOGMETRICREQUEST'].fields_by_name['metric']._serialized_options = b'\340A\002'
  _globals['_DELETELOGMETRICREQUEST'].fields_by_name['metric_name']._loaded_options = None
  _globals['_DELETELOGMETRICREQUEST'].fields_by_name['metric_name']._serialized_options = b'\340A\002\372A\"\n logging.googleapis.com/LogMetric'
  _globals['_METRICSSERVICEV2']._loaded_options = None
  _globals['_METRICSSERVICEV2']._serialized_options = b'\312A\026logging.googleapis.com\322A\360\001https://www.googleapis.com/auth/cloud-platform,https://www.googleapis.com/auth/cloud-platform.read-only,https://www.googleapis.com/auth/logging.admin,https://www.googleapis.com/auth/logging.read,https://www.googleapis.com/auth/logging.write'
  _globals['_METRICSSERVICEV2'].methods_by_name['ListLogMetrics']._loaded_options = None
  _globals['_METRICSSERVICEV2'].methods_by_name['ListLogMetrics']._serialized_options = b'\332A\006parent\202\323\344\223\002!\022\037/v2/{parent=projects/*}/metrics'
  _globals['_METRICSSERVICEV2'].methods_by_name['GetLogMetric']._loaded_options = None
  _globals['_METRICSSERVICEV2'].methods_by_name['GetLogMetric']._serialized_options = b'\332A\013metric_name\202\323\344\223\002(\022&/v2/{metric_name=projects/*/metrics/*}'
  _globals['_METRICSSERVICEV2'].methods_by_name['CreateLogMetric']._loaded_options = None
  _globals['_METRICSSERVICEV2'].methods_by_name['CreateLogMetric']._serialized_options = b'\332A\rparent,metric\202\323\344\223\002)\"\037/v2/{parent=projects/*}/metrics:\006metric'
  _globals['_METRICSSERVICEV2'].methods_by_name['UpdateLogMetric']._loaded_options = None
  _globals['_METRICSSERVICEV2'].methods_by_name['UpdateLogMetric']._serialized_options = b'\332A\022metric_name,metric\202\323\344\223\0020\032&/v2/{metric_name=projects/*/metrics/*}:\006metric'
  _globals['_METRICSSERVICEV2'].methods_by_name['DeleteLogMetric']._loaded_options = None
  _globals['_METRICSSERVICEV2'].methods_by_name['DeleteLogMetric']._serialized_options = b'\332A\013metric_name\202\323\344\223\002(*&/v2/{metric_name=projects/*/metrics/*}'
  _globals['_LOGMETRIC']._serialized_start=296
  _globals['_LOGMETRIC']._serialized_end=997
  _globals['_LOGMETRIC_LABELEXTRACTORSENTRY']._serialized_start=837
  _globals['_LOGMETRIC_LABELEXTRACTORSENTRY']._serialized_end=891
  _globals['_LOGMETRIC_APIVERSION']._serialized_start=893
  _globals['_LOGMETRIC_APIVERSION']._serialized_end=921
  _globals['_LISTLOGMETRICSREQUEST']._serialized_start=1000
  _globals['_LISTLOGMETRICSREQUEST']._serialized_end=1141
  _globals['_LISTLOGMETRICSRESPONSE']._serialized_start=1143
  _globals['_LISTLOGMETRICSRESPONSE']._serialized_end=1239
  _globals['_GETLOGMETRICREQUEST']._serialized_start=1241
  _globals['_GETLOGMETRICREQUEST']._serialized_end=1325
  _globals['_CREATELOGMETRICREQUEST']._serialized_start=1328
  _globals['_CREATELOGMETRICREQUEST']._serialized_end=1461
  _globals['_UPDATELOGMETRICREQUEST']._serialized_start=1464
  _globals['_UPDATELOGMETRICREQUEST']._serialized_end=1602
  _globals['_DELETELOGMETRICREQUEST']._serialized_start=1604
  _globals['_DELETELOGMETRICREQUEST']._serialized_end=1691
  _globals['_METRICSSERVICEV2']._serialized_start=1694
  _globals['_METRICSSERVICEV2']._serialized_end=2764
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/service/accesslog/v3/als.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/service/accesslog/v3/als.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import base_pb2 as envoy_dot_config_dot_core_dot_v3_dot_base__pb2
from envoy.data.accesslog.v3 import accesslog_pb2 as envoy_dot_data_dot_accesslog_dot_v3_dot_accesslog__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$envoy/service/accesslog/v3/als.proto\x12\x1a\x65nvoy.service.accesslog.v3\x1a\x1f\x65nvoy/config/core/v3/base.proto\x1a\'envoy/data/accesslog/v3/accesslog.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"V\n\x18StreamAccessLogsResponse::\x9a\xc5\x88\x1e\x35\n3envoy.service.accesslog.v2.StreamAccessLogsResponse\"\xff\x06\n\x17StreamAccessLogsMessage\x12R\n\nidentifier\x18\x01 \x01(\x0b\x32>.envoy.service.accesslog.v3.StreamAccessLogsMessage.Identifier\x12]\n\thttp_logs\x18\x02 \x01(\x0b\x32H.envoy.service.accesslog.v3.StreamAccessLogsMessage.HTTPAccessLogEntriesH\x00\x12[\n\x08tcp_logs\x18\x03 \x01(\x0b\x32G.envoy.service.accesslog.v3.StreamAccessLogsMessage.TCPAccessLogEntriesH\x00\x1a\xa1\x01\n\nIdentifier\x12\x32\n\x04node\x18\x01 \x01(\x0b\x32\x1a.envoy.config.core.v3.NodeB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12\x19\n\x08log_name\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01:D\x9a\xc5\x88\x1e?\n=envoy.service.accesslog.v2.StreamAccessLogsMessage.Identifier\x1a\xb0\x01\n\x14HTTPAccessLogEntries\x12H\n\tlog_entry\x18\x01 \x03(\x0b\x32+.envoy.data.accesslog.v3.HTTPAccessLogEntryB\x08\xfa\x42\x05\x92\x01\x02\x08\x01:N\x9a\xc5\x88\x1eI\nGenvoy.service.accesslog.v2.StreamAccessLogsMessage.HTTPAccessLogEntries\x1a\xad\x01\n\x13TCPAccessLogEntries\x12G\n\tlog_entry\x18\x01 \x03(\x0b\x32*.envoy.data.accesslog.v3.TCPAccessLogEntryB\x08\xfa\x42\x05\x92\x01\x02\x08\x01:M\x9a\xc5\x88\x1eH\nFenvoy.service.accesslog.v2.StreamAccessLogsMessage.TCPAccessLogEntries:9\x9a\xc5\x88\x1e\x34\n2envoy.service.accesslog.v2.StreamAccessLogsMessageB\x12\n\x0blog_entries\x12\x03\xf8\x42\x01\x32\x96\x01\n\x10\x41\x63\x63\x65ssLogService\x12\x81\x01\n\x10StreamAccessLogs\x12\x33.envoy.service.accesslog.v3.StreamAccessLogsMessage\x1a\x34.envoy.service.accesslog.v3.StreamAccessLogsResponse\"\x00(\x01\x42\x8d\x01\n(io.envoyproxy.envoy.service.accesslog.v3B\x08\x41lsProtoP\x01ZMgithub.com/envoyproxy/go-control-plane/envoy/service/accesslog/v3;accesslogv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.service.accesslog.v3.als_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n(io.envoyproxy.envoy.service.accesslog.v3B\010AlsProtoP\001ZMgithub.com/envoyproxy/go-control-plane/envoy/service/accesslog/v3;accesslogv3\272\200\310\321\006\002\020\002'
  _globals['_STREAMACCESSLOGSRESPONSE']._loaded_options = None
  _globals['_STREAMACCESSLOGSRESPONSE']._serialized_options = b'\232\305\210\0365\n3envoy.service.accesslog.v2.StreamAccessLogsResponse'
  _globals['_STREAMACCESSLOGSMESSAGE_IDENTIFIER'].fields_by_name['node']._loaded_options = None
  _globals['_STREAMACCESSLOGSMESSAGE_IDENTIFIER'].fields_by_name['node']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_STREAMACCESSLOGSMESSAGE_IDENTIFIER'].fields_by_name['log_name']._loaded_options = None
  _globals['_STREAMACCESSLOGSMESSAGE_IDENTIFIER'].fields_by_name['log_name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_STREAMACCESSLOGSMESSAGE_IDENTIFIER']._loaded_options = None
  _globals['_STREAMACCESSLOGSMESSAGE_IDENTIFIER']._serialized_options = b'\232\305\210\036?\n=envoy.service.accesslog.v2.StreamAccessLogsMessage.Identifier'
  _globals['_STREAMACCESSLOGSMESSAGE_HTTPACCESSLOGENTRIES'].fields_by_name['log_entry']._loaded_options = None
  _globals['_STREAMACCESSLOGSMESSAGE_HTTPACCESSLOGENTRIES'].fields_by_name['log_entry']._serialized_options = b'\372B\005\222\001\002\010\001'
  _globals['_STREAMACCESSLOGSMESSAGE_HTTPACCESSLOGENTRIES']._loaded_options = None
  _globals['_STREAMACCESSLOGSMESSAGE_HTTPACCESSLOGENTRIES']._serialized_options = b'\232\305\210\036I\nGenvoy.service.accesslog.v2.StreamAccessLogsMessage.HTTPAccessLogEntries'
  _globals['_STREAMACCESSLOGSMESSAGE_TCPACCESSLOGENTRIES'].fields_by_name['log_entry']._loaded_options = None
  _globals['_STREAMACCESSLOGSMESSAGE_TCPACCESSLOGENTRIES'].fields_by_name['log_entry']._serialized_options = b'\372B\005\222\001\002\010\001'
  _globals['_STREAMACCESSLOGSMESSAGE_TCPACCESSLOGENTRIES']._loaded_options = None
  _globals['_STREAMACCESSLOGSMESSAGE_TCPACCESSLOGENTRIES']._serialized_options = b'\232\305\210\036H\nFenvoy.service.accesslog.v2.StreamAccessLogsMessage.TCPAccessLogEntries'
  _globals['_STREAMACCESSLOGSMESSAGE'].oneofs_by_name['log_entries']._loaded_options = None
  _globals['_STREAMACCESSLOGSMESSAGE'].oneofs_by_name['log_entries']._serialized_options = b'\370B\001'
  _globals['_STREAMACCESSLOGSMESSAGE']._loaded_options = None
  _globals['_STREAMACCESSLOGSMESSAGE']._serialized_options = b'\232\305\210\0364\n2envoy.service.accesslog.v2.StreamAccessLogsMessage'
  _globals['_STREAMACCESSLOGSRESPONSE']._serialized_start=233
  _globals['_STREAMACCESSLOGSRESPONSE']._serialized_end=319
  _globals['_STREAMACCESSLOGSMESSAGE']._serialized_start=322
  _globals['_STREAMACCESSLOGSMESSAGE']._serialized_end=1217
  _globals['_STREAMACCESSLOGSMESSAGE_IDENTIFIER']._serialized_start=622
  _globals['_STREAMACCESSLOGSMESSAGE_IDENTIFIER']._serialized_end=783
  _globals['_STREAMACCESSLOGSMESSAGE_HTTPACCESSLOGENTRIES']._serialized_start=786
  _globals['_STREAMACCESSLOGSMESSAGE_HTTPACCESSLOGENTRIES']._serialized_end=962
  _globals['_STREAMACCESSLOGSMESSAGE_TCPACCESSLOGENTRIES']._serialized_start=965
  _globals['_STREAMACCESSLOGSMESSAGE_TCPACCESSLOGENTRIES']._serialized_end=1138
  _globals['_ACCESSLOGSERVICE']._serialized_start=1220
  _globals['_ACCESSLOGSERVICE']._serialized_end=1370
# @@protoc_insertion_point(module_scope)



# Copyright 2023 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The Python implementation of the GRPC helloworld.Greeter server with health checking."""

from concurrent import futures
import logging
import threading
from time import sleep

import grpc
from grpc_health.v1 import health
from grpc_health.v1 import health_pb2
from grpc_health.v1 import health_pb2_grpc
import helloworld_pb2
import helloworld_pb2_grpc


class Greeter(helloworld_pb2_grpc.GreeterServicer):
    def SayHello(self, request, context):
        return helloworld_pb2.HelloReply(message=request.name)


def _toggle_health(health_servicer: health.HealthServicer, service: str):
    next_status = health_pb2.HealthCheckResponse.SERVING
    while True:
        if next_status == health_pb2.HealthCheckResponse.SERVING:
            next_status = health_pb2.HealthCheckResponse.NOT_SERVING
        else:
            next_status = health_pb2.HealthCheckResponse.SERVING

        health_servicer.set(service, next_status)
        sleep(5)


def _configure_health_server(server: grpc.Server):
    health_servicer = health.HealthServicer(
        experimental_non_blocking=True,
        experimental_thread_pool=futures.ThreadPoolExecutor(max_workers=10),
    )
    health_pb2_grpc.add_HealthServicer_to_server(health_servicer, server)

    # Use a daemon thread to toggle health status
    toggle_health_status_thread = threading.Thread(
        target=_toggle_health,
        args=(health_servicer, "helloworld.Greeter"),
        daemon=True,
    )
    toggle_health_status_thread.start()


def serve():
    port = "50051"
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
    helloworld_pb2_grpc.add_GreeterServicer_to_server(Greeter(), server)
    server.add_insecure_port("[::]:" + port)
    _configure_health_server(server)
    server.start()
    print("Server started, listening on " + port)
    server.wait_for_termination()


if __name__ == "__main__":
    logging.basicConfig()
    serve()



#!/usr/bin/env python3

# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Read from stdin a set of colon separated http headers:

    :path: /foo/bar
    content-type: application/grpc

Write a set of strings containing a hpack encoded http2 frame that
represents said headers.
"""

import argparse
import json
import sys


def append_never_indexed(payload_line, n, count, key, value, value_is_huff):
    payload_line.append(0x10)
    assert len(key) <= 126
    payload_line.append(len(key))
    payload_line.extend(ord(c) for c in key)
    assert len(value) <= 126
    payload_line.append(len(value) + (0x80 if value_is_huff else 0))
    payload_line.extend(value)


def append_inc_indexed(payload_line, n, count, key, value, value_is_huff):
    payload_line.append(0x40)
    assert len(key) <= 126
    payload_line.append(len(key))
    payload_line.extend(ord(c) for c in key)
    assert len(value) <= 126
    payload_line.append(len(value) + (0x80 if value_is_huff else 0))
    payload_line.extend(value)


def append_pre_indexed(payload_line, n, count, key, value, value_is_huff):
    assert not value_is_huff
    payload_line.append(0x80 + 61 + count - n)


def esc_c(line):
    out = '"'
    last_was_hex = False
    for c in line:
        if 32 <= c < 127:
            if c in hex_bytes and last_was_hex:
                out += '""'
            if c != ord('"'):
                out += chr(c)
            else:
                out += '\\"'
            last_was_hex = False
        else:
            out += "\\x%02x" % c
            last_was_hex = True
    return out + '"'


def output_c(payload_bytes):
    for line in payload_bytes:
        print((esc_c(line)))


def output_hex(payload_bytes):
    all_bytes = []
    for line in payload_bytes:
        all_bytes.extend(line)
    print(("{%s}" % ", ".join("0x%02x" % c for c in all_bytes)))


def output_hexstr(payload_bytes):
    all_bytes = []
    for line in payload_bytes:
        all_bytes.extend(line)
    print(("%s" % "".join("%02x" % c for c in all_bytes)))


_COMPRESSORS = {
    "never": append_never_indexed,
    "inc": append_inc_indexed,
    "pre": append_pre_indexed,
}

_OUTPUTS = {
    "c": output_c,
    "hex": output_hex,
    "hexstr": output_hexstr,
}

argp = argparse.ArgumentParser("Generate header frames")
argp.add_argument(
    "--set_end_stream", default=False, action="store_const", const=True
)
argp.add_argument(
    "--no_framing", default=False, action="store_const", const=True
)
argp.add_argument(
    "--compression", choices=sorted(_COMPRESSORS.keys()), default="never"
)
argp.add_argument("--huff", default=False, action="store_const", const=True)
argp.add_argument("--output", default="c", choices=sorted(_OUTPUTS.keys()))
args = argp.parse_args()

# parse input, fill in vals
vals = []
for line in sys.stdin:
    line = line.strip()
    if line == "":
        continue
    if line[0] == "#":
        continue
    key_tail, value = line[1:].split(":")
    key = (line[0] + key_tail).strip()
    value = value.strip().encode("ascii")
    if args.huff:
        from hpack.huffman import HuffmanEncoder
        from hpack.huffman_constants import REQUEST_CODES
        from hpack.huffman_constants import REQUEST_CODES_LENGTH

        value = HuffmanEncoder(REQUEST_CODES, REQUEST_CODES_LENGTH).encode(
            value
        )
    vals.append((key, value))

# generate frame payload binary data
payload_bytes = []
if not args.no_framing:
    payload_bytes.append([])  # reserve space for header
payload_len = 0
n = 0
for key, value in vals:
    payload_line = []
    _COMPRESSORS[args.compression](
        payload_line, n, len(vals), key, value, args.huff
    )
    n += 1
    payload_len += len(payload_line)
    payload_bytes.append(payload_line)

# fill in header
if not args.no_framing:
    flags = 0x04  # END_HEADERS
    if args.set_end_stream:
        flags |= 0x01  # END_STREAM
    payload_bytes[0].extend(
        [
            (payload_len >> 16) & 0xFF,
            (payload_len >> 8) & 0xFF,
            (payload_len) & 0xFF,
            # header frame
            0x01,
            # flags
            flags,
            # stream id
            0x00,
            0x00,
            0x00,
            0x01,
        ]
    )

hex_bytes = [ord(c) for c in "abcdefABCDEF0123456789"]

# dump bytes
_OUTPUTS[args.output](payload_bytes)



# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""An example gRPC Python-using application's common code elements."""

from tests.testing.proto import requests_pb2
from tests.testing.proto import services_pb2

SERVICE_NAME = "tests_of_grpc_testing.FirstService"
UNARY_UNARY_METHOD_NAME = "UnUn"
UNARY_STREAM_METHOD_NAME = "UnStre"
STREAM_UNARY_METHOD_NAME = "StreUn"
STREAM_STREAM_METHOD_NAME = "StreStre"

UNARY_UNARY_REQUEST = requests_pb2.Up(first_up_field=2)
ERRONEOUS_UNARY_UNARY_REQUEST = requests_pb2.Up(first_up_field=3)
UNARY_UNARY_RESPONSE = services_pb2.Down(first_down_field=5)
ERRONEOUS_UNARY_UNARY_RESPONSE = services_pb2.Down(first_down_field=7)
UNARY_STREAM_REQUEST = requests_pb2.Charm(first_charm_field=11)
STREAM_UNARY_REQUEST = requests_pb2.Charm(first_charm_field=13)
STREAM_UNARY_RESPONSE = services_pb2.Strange(first_strange_field=17)
STREAM_STREAM_REQUEST = requests_pb2.Top(first_top_field=19)
STREAM_STREAM_RESPONSE = services_pb2.Bottom(first_bottom_field=23)
TWO_STREAM_STREAM_RESPONSES = (STREAM_STREAM_RESPONSE,) * 2
ABORT_REQUEST = requests_pb2.Up(first_up_field=42)
ABORT_SUCCESS_QUERY = requests_pb2.Up(first_up_field=43)
ABORT_NO_STATUS_RESPONSE = services_pb2.Down(first_down_field=50)
ABORT_SUCCESS_RESPONSE = services_pb2.Down(first_down_field=51)
ABORT_FAILURE_RESPONSE = services_pb2.Down(first_down_field=52)
STREAM_STREAM_MUTATING_REQUEST = requests_pb2.Top(first_top_field=24601)
STREAM_STREAM_MUTATING_COUNT = 2

INFINITE_REQUEST_STREAM_TIMEOUT = 0.2



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests of grpc_health.v1.health."""

import logging
import queue
import threading
import time
import unittest

import grpc
from grpc_health.v1 import health
from grpc_health.v1 import health_pb2
from grpc_health.v1 import health_pb2_grpc

from tests.unit import test_common
from tests.unit import thread_pool
from tests.unit.framework.common import test_constants

_SERVING_SERVICE = "grpc.test.TestServiceServing"
_UNKNOWN_SERVICE = "grpc.test.TestServiceUnknown"
_NOT_SERVING_SERVICE = "grpc.test.TestServiceNotServing"
_WATCH_SERVICE = "grpc.test.WatchService"


def _consume_responses(response_iterator, response_queue):
    for response in response_iterator:
        response_queue.put(response)


class BaseWatchTests:
    class WatchTests(unittest.TestCase):
        def start_server(self, non_blocking=False, thread_pool=None):
            self._thread_pool = thread_pool
            self._servicer = health.HealthServicer(
                experimental_non_blocking=non_blocking,
                experimental_thread_pool=thread_pool,
            )
            self._servicer.set(
                _SERVING_SERVICE, health_pb2.HealthCheckResponse.SERVING
            )
            self._servicer.set(
                _UNKNOWN_SERVICE, health_pb2.HealthCheckResponse.UNKNOWN
            )
            self._servicer.set(
                _NOT_SERVING_SERVICE, health_pb2.HealthCheckResponse.NOT_SERVING
            )
            self._server = test_common.test_server()
            port = self._server.add_insecure_port("[::]:0")
            health_pb2_grpc.add_HealthServicer_to_server(
                self._servicer, self._server
            )
            self._server.start()

            self._channel = grpc.insecure_channel("localhost:%d" % port)
            self._stub = health_pb2_grpc.HealthStub(self._channel)

        def tearDown(self):
            self._server.stop(None)
            self._channel.close()

        def test_watch_empty_service(self):
            request = health_pb2.HealthCheckRequest(service="")
            response_queue = queue.Queue()
            rendezvous = self._stub.Watch(request)
            thread = threading.Thread(
                target=_consume_responses, args=(rendezvous, response_queue)
            )
            thread.start()

            response = response_queue.get(timeout=test_constants.SHORT_TIMEOUT)
            self.assertEqual(
                health_pb2.HealthCheckResponse.SERVING, response.status
            )

            rendezvous.cancel()
            thread.join()
            self.assertTrue(response_queue.empty())

            if self._thread_pool is not None:
                self.assertTrue(self._thread_pool.was_used())

        def test_watch_new_service(self):
            request = health_pb2.HealthCheckRequest(service=_WATCH_SERVICE)
            response_queue = queue.Queue()
            rendezvous = self._stub.Watch(request)
            thread = threading.Thread(
                target=_consume_responses, args=(rendezvous, response_queue)
            )
            thread.start()

            response = response_queue.get(timeout=test_constants.SHORT_TIMEOUT)
            self.assertEqual(
                health_pb2.HealthCheckResponse.SERVICE_UNKNOWN, response.status
            )

            self._servicer.set(
                _WATCH_SERVICE, health_pb2.HealthCheckResponse.SERVING
            )
            response = response_queue.get(timeout=test_constants.SHORT_TIMEOUT)
            self.assertEqual(
                health_pb2.HealthCheckResponse.SERVING, response.status
            )

            self._servicer.set(
                _WATCH_SERVICE, health_pb2.HealthCheckResponse.NOT_SERVING
            )
            response = response_queue.get(timeout=test_constants.SHORT_TIMEOUT)
            self.assertEqual(
                health_pb2.HealthCheckResponse.NOT_SERVING, response.status
            )

            rendezvous.cancel()
            thread.join()
            self.assertTrue(response_queue.empty())

        def test_watch_service_isolation(self):
            request = health_pb2.HealthCheckRequest(service=_WATCH_SERVICE)
            response_queue = queue.Queue()
            rendezvous = self._stub.Watch(request)
            thread = threading.Thread(
                target=_consume_responses, args=(rendezvous, response_queue)
            )
            thread.start()

            response = response_queue.get(timeout=test_constants.SHORT_TIMEOUT)
            self.assertEqual(
                health_pb2.HealthCheckResponse.SERVICE_UNKNOWN, response.status
            )

            self._servicer.set(
                "some-other-service", health_pb2.HealthCheckResponse.SERVING
            )
            with self.assertRaises(queue.Empty):
                response_queue.get(timeout=test_constants.SHORT_TIMEOUT)

            rendezvous.cancel()
            thread.join()
            self.assertTrue(response_queue.empty())

        def test_two_watchers(self):
            request = health_pb2.HealthCheckRequest(service=_WATCH_SERVICE)
            response_queue1 = queue.Queue()
            response_queue2 = queue.Queue()
            rendezvous1 = self._stub.Watch(request)
            rendezvous2 = self._stub.Watch(request)
            thread1 = threading.Thread(
                target=_consume_responses, args=(rendezvous1, response_queue1)
            )
            thread2 = threading.Thread(
                target=_consume_responses, args=(rendezvous2, response_queue2)
            )
            thread1.start()
            thread2.start()

            response1 = response_queue1.get(
                timeout=test_constants.SHORT_TIMEOUT
            )
            response2 = response_queue2.get(
                timeout=test_constants.SHORT_TIMEOUT
            )
            self.assertEqual(
                health_pb2.HealthCheckResponse.SERVICE_UNKNOWN, response1.status
            )
            self.assertEqual(
                health_pb2.HealthCheckResponse.SERVICE_UNKNOWN, response2.status
            )

            self._servicer.set(
                _WATCH_SERVICE, health_pb2.HealthCheckResponse.SERVING
            )
            response1 = response_queue1.get(
                timeout=test_constants.SHORT_TIMEOUT
            )
            response2 = response_queue2.get(
                timeout=test_constants.SHORT_TIMEOUT
            )
            self.assertEqual(
                health_pb2.HealthCheckResponse.SERVING, response1.status
            )
            self.assertEqual(
                health_pb2.HealthCheckResponse.SERVING, response2.status
            )

            rendezvous1.cancel()
            rendezvous2.cancel()
            thread1.join()
            thread2.join()
            self.assertTrue(response_queue1.empty())
            self.assertTrue(response_queue2.empty())

        @unittest.skip("https://github.com/grpc/grpc/issues/18127")
        def test_cancelled_watch_removed_from_watch_list(self):
            request = health_pb2.HealthCheckRequest(service=_WATCH_SERVICE)
            response_queue = queue.Queue()
            rendezvous = self._stub.Watch(request)
            thread = threading.Thread(
                target=_consume_responses, args=(rendezvous, response_queue)
            )
            thread.start()

            response = response_queue.get(timeout=test_constants.SHORT_TIMEOUT)
            self.assertEqual(
                health_pb2.HealthCheckResponse.SERVICE_UNKNOWN, response.status
            )

            rendezvous.cancel()
            self._servicer.set(
                _WATCH_SERVICE, health_pb2.HealthCheckResponse.SERVING
            )
            thread.join()

            # Wait, if necessary, for serving thread to process client cancellation
            timeout = time.time() + test_constants.TIME_ALLOWANCE
            while (
                time.time() < timeout
                and self._servicer._send_response_callbacks[_WATCH_SERVICE]
            ):
                time.sleep(1)
            self.assertFalse(
                self._servicer._send_response_callbacks[_WATCH_SERVICE],
                "watch set should be empty",
            )
            self.assertTrue(response_queue.empty())

        def test_graceful_shutdown(self):
            request = health_pb2.HealthCheckRequest(service="")
            response_queue = queue.Queue()
            rendezvous = self._stub.Watch(request)
            thread = threading.Thread(
                target=_consume_responses, args=(rendezvous, response_queue)
            )
            thread.start()

            response = response_queue.get(timeout=test_constants.SHORT_TIMEOUT)
            self.assertEqual(
                health_pb2.HealthCheckResponse.SERVING, response.status
            )

            self._servicer.enter_graceful_shutdown()
            response = response_queue.get(timeout=test_constants.SHORT_TIMEOUT)
            self.assertEqual(
                health_pb2.HealthCheckResponse.NOT_SERVING, response.status
            )

            # This should be a no-op.
            self._servicer.set("", health_pb2.HealthCheckResponse.SERVING)

            rendezvous.cancel()
            thread.join()
            self.assertTrue(response_queue.empty())


class HealthServicerTest(BaseWatchTests.WatchTests):
    def setUp(self):
        self._thread_pool = thread_pool.RecordingThreadPool(max_workers=None)
        super(HealthServicerTest, self).start_server(
            non_blocking=True, thread_pool=self._thread_pool
        )

    def test_check_empty_service(self):
        request = health_pb2.HealthCheckRequest()
        resp = self._stub.Check(request)
        self.assertEqual(health_pb2.HealthCheckResponse.SERVING, resp.status)

    def test_check_serving_service(self):
        request = health_pb2.HealthCheckRequest(service=_SERVING_SERVICE)
        resp = self._stub.Check(request)
        self.assertEqual(health_pb2.HealthCheckResponse.SERVING, resp.status)

    def test_check_unknown_service(self):
        request = health_pb2.HealthCheckRequest(service=_UNKNOWN_SERVICE)
        resp = self._stub.Check(request)
        self.assertEqual(health_pb2.HealthCheckResponse.UNKNOWN, resp.status)

    def test_check_not_serving_service(self):
        request = health_pb2.HealthCheckRequest(service=_NOT_SERVING_SERVICE)
        resp = self._stub.Check(request)
        self.assertEqual(
            health_pb2.HealthCheckResponse.NOT_SERVING, resp.status
        )

    def test_check_not_found_service(self):
        request = health_pb2.HealthCheckRequest(service="not-found")
        with self.assertRaises(grpc.RpcError) as context:
            resp = self._stub.Check(request)

        self.assertEqual(grpc.StatusCode.NOT_FOUND, context.exception.code())

    def test_health_service_name(self):
        self.assertEqual(health.SERVICE_NAME, "grpc.health.v1.Health")


class HealthServicerBackwardsCompatibleWatchTest(BaseWatchTests.WatchTests):
    def setUp(self):
        super(HealthServicerBackwardsCompatibleWatchTest, self).start_server(
            non_blocking=False, thread_pool=None
        )


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=2)



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests of channel arguments on client/server side."""

from concurrent import futures
import logging
import unittest

import grpc


class TestPointerWrapper:
    def __int__(self):
        return 123456


TEST_CHANNEL_ARGS = (
    ("arg1", b"bytes_val"),
    ("arg2", "str_val"),
    ("arg3", 1),
    (b"arg4", "str_val"),
    ("arg6", TestPointerWrapper()),
)

INVALID_TEST_CHANNEL_ARGS = [
    {"foo": "bar"},
    (("key",),),
    "str",
]


class ChannelArgsTest(unittest.TestCase):
    def test_client(self):
        grpc.insecure_channel("localhost:8080", options=TEST_CHANNEL_ARGS)

    def test_server(self):
        grpc.server(
            futures.ThreadPoolExecutor(max_workers=1), options=TEST_CHANNEL_ARGS
        )

    def test_invalid_client_args(self):
        for invalid_arg in INVALID_TEST_CHANNEL_ARGS:
            self.assertRaises(
                ValueError,
                grpc.insecure_channel,
                "localhost:8080",
                options=invalid_arg,
            )


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=2)



#!/usr/bin/env python3

# Copyright 2023 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys


# utility: print a big comment block into a set of files
def put_banner(files, banner):
    for f in files:
        for line in banner:
            print("// %s" % line, file=f)
        print("", file=f)


with open("src/core/util/if_list.h", "w") as H:
    # copy-paste copyright notice from this file
    with open(sys.argv[0]) as my_source:
        copyright = []
        for line in my_source:
            if line[0] != "#":
                break
        for line in my_source:
            if line[0] == "#":
                copyright.append(line)
                break
        for line in my_source:
            if line[0] != "#":
                break
            copyright.append(line)
        put_banner([H], [line[2:].rstrip() for line in copyright])

    put_banner([H], ["", "Automatically generated by %s" % sys.argv[0], ""])

    print("#ifndef GRPC_CORE_UTIL_IF_LIST_H", file=H)
    print("#define GRPC_CORE_UTIL_IF_LIST_H", file=H)
    print("", file=H)
    print("#include <grpc/support/port_platform.h>", file=H)
    print("", file=H)
    print("#include <stdlib.h>", file=H)
    print("", file=H)
    print("namespace grpc_core {", file=H)

    for n in range(1, 64):
        print("", file=H)
        print(
            "template <typename CheckArg, typename ActionArg, typename"
            " ActionFail, %s, %s> auto IfList(CheckArg input, ActionArg"
            " action_arg, ActionFail action_fail, %s, %s) {"
            % (
                ", ".join("typename Check%d" % (i) for i in range(0, n)),
                ", ".join("typename Action%d" % (i) for i in range(0, n)),
                ", ".join("Check%d check%d" % (i, i) for i in range(0, n)),
                ", ".join("Action%d action%d" % (i, i) for i in range(0, n)),
            ),
            file=H,
        )
        for i in range(0, n):
            print(
                "  if (check%d(input)) return action%d(action_arg);" % (i, i),
                file=H,
            )
        print("  return action_fail(action_arg);", file=H)
        print("}", file=H)

    print("", file=H)
    print("}", file=H)
    print("", file=H)
    print("#endif // GRPC_CORE_UTIL_IF_LIST_H", file=H)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/http/mcp/v3/mcp.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/http/mcp/v3/mcp.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n.envoy/extensions/filters/http/mcp/v3/mcp.proto\x12$envoy.extensions.filters.http.mcp.v3\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xbf\x02\n\x03Mcp\x12U\n\x0ctraffic_mode\x18\x01 \x01(\x0e\x32\x35.envoy.extensions.filters.http.mcp.v3.Mcp.TrafficModeB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12\x19\n\x11\x63lear_route_cache\x18\x02 \x01(\x08\x12G\n\x15max_request_body_size\x18\x03 \x01(\x0b\x32\x1c.google.protobuf.UInt32ValueB\n\xfa\x42\x07*\x05\x18\x80\x80\x80\x05\x12I\n\rparser_config\x18\x04 \x01(\x0b\x32\x32.envoy.extensions.filters.http.mcp.v3.ParserConfig\"2\n\x0bTrafficMode\x12\x10\n\x0cPASS_THROUGH\x10\x00\x12\x11\n\rREJECT_NO_MCP\x10\x01\"\xa2\x02\n\x0cParserConfig\x12P\n\x07methods\x18\x01 \x03(\x0b\x32?.envoy.extensions.filters.http.mcp.v3.ParserConfig.MethodConfig\x1a\x30\n\x17\x41ttributeExtractionRule\x12\x15\n\x04path\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x1a\x8d\x01\n\x0cMethodConfig\x12\x17\n\x06method\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x64\n\x10\x65xtraction_rules\x18\x02 \x03(\x0b\x32J.envoy.extensions.filters.http.mcp.v3.ParserConfig.AttributeExtractionRule\"\xad\x01\n\x0bMcpOverride\x12U\n\x0ctraffic_mode\x18\x01 \x01(\x0e\x32\x35.envoy.extensions.filters.http.mcp.v3.Mcp.TrafficModeB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12G\n\x15max_request_body_size\x18\x02 \x01(\x0b\x32\x1c.google.protobuf.UInt32ValueB\n\xfa\x42\x07*\x05\x18\x80\x80\x80\x05\x42\xa3\x01\n2io.envoyproxy.envoy.extensions.filters.http.mcp.v3B\x08McpProtoP\x01ZQgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/mcp/v3;mcpv3\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.http.mcp.v3.mcp_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n2io.envoyproxy.envoy.extensions.filters.http.mcp.v3B\010McpProtoP\001ZQgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/mcp/v3;mcpv3\272\200\310\321\006\002\020\002\322\306\244\341\006\002\010\001'
  _globals['_MCP'].fields_by_name['traffic_mode']._loaded_options = None
  _globals['_MCP'].fields_by_name['traffic_mode']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_MCP'].fields_by_name['max_request_body_size']._loaded_options = None
  _globals['_MCP'].fields_by_name['max_request_body_size']._serialized_options = b'\372B\007*\005\030\200\200\200\005'
  _globals['_PARSERCONFIG_ATTRIBUTEEXTRACTIONRULE'].fields_by_name['path']._loaded_options = None
  _globals['_PARSERCONFIG_ATTRIBUTEEXTRACTIONRULE'].fields_by_name['path']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_PARSERCONFIG_METHODCONFIG'].fields_by_name['method']._loaded_options = None
  _globals['_PARSERCONFIG_METHODCONFIG'].fields_by_name['method']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_MCPOVERRIDE'].fields_by_name['traffic_mode']._loaded_options = None
  _globals['_MCPOVERRIDE'].fields_by_name['traffic_mode']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_MCPOVERRIDE'].fields_by_name['max_request_body_size']._loaded_options = None
  _globals['_MCPOVERRIDE'].fields_by_name['max_request_body_size']._serialized_options = b'\372B\007*\005\030\200\200\200\005'
  _globals['_MCP']._serialized_start=210
  _globals['_MCP']._serialized_end=529
  _globals['_MCP_TRAFFICMODE']._serialized_start=479
  _globals['_MCP_TRAFFICMODE']._serialized_end=529
  _globals['_PARSERCONFIG']._serialized_start=532
  _globals['_PARSERCONFIG']._serialized_end=822
  _globals['_PARSERCONFIG_ATTRIBUTEEXTRACTIONRULE']._serialized_start=630
  _globals['_PARSERCONFIG_ATTRIBUTEEXTRACTIONRULE']._serialized_end=678
  _globals['_PARSERCONFIG_METHODCONFIG']._serialized_start=681
  _globals['_PARSERCONFIG_METHODCONFIG']._serialized_end=822
  _globals['_MCPOVERRIDE']._serialized_start=825
  _globals['_MCPOVERRIDE']._serialized_end=998
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python3

# Copyright 2020 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# This is based on the script on the Envoy project
# https://github.com/envoyproxy/envoy/blob/master/tools/gen_compilation_database.py

import argparse
import glob
import json
import logging
import os
from pathlib import Path
import re
import shlex
import subprocess

RE_INCLUDE_SYSTEM = re.compile("\s*-I\s+/usr/[^ ]+")


# This method is equivalent to https://github.com/grailbio/bazel-compilation-database/blob/master/generate.sh
def generateCompilationDatabase(args):
    # We need to download all remote outputs for generated source code.
    # This option lives here to override those specified in bazelrc.
    bazel_options = shlex.split(os.environ.get("BAZEL_BUILD_OPTIONS", "")) + [
        "--config=compdb",
        "--remote_download_outputs=all",
    ]

    subprocess.check_call(
        ["bazel", "build"]
        + bazel_options
        + [
            "--aspects=@bazel_compdb//:aspects.bzl%compilation_database_aspect",
            "--output_groups=compdb_files,header_files",
        ]
        + args.bazel_targets
    )

    execroot = (
        subprocess.check_output(
            ["bazel", "info", "execution_root"] + bazel_options
        )
        .decode()
        .strip()
    )

    compdb = []
    for compdb_file in Path(execroot).glob("**/*.compile_commands.json"):
        compdb.extend(
            json.loads(
                compdb_file.read_text().replace("__EXEC_ROOT__", execroot)
            )
        )

    if args.dedup_targets:
        compdb_map = {target["file"]: target for target in compdb}
        compdb = list(compdb_map.values())

    return compdb


def isHeader(filename):
    for ext in (".h", ".hh", ".hpp", ".hxx"):
        if filename.endswith(ext):
            return True
    return False


def isCompileTarget(target, args):
    filename = target["file"]
    if not args.include_headers and isHeader(filename):
        return False
    if not args.include_genfiles:
        if filename.startswith("bazel-out/"):
            return False
    if not args.include_external:
        if filename.startswith("external/"):
            return False
    return True


def modifyCompileCommand(target, args):
    cc, options = target["command"].split(" ", 1)

    # Workaround for bazel added C++14 options, those don't affect the build itself but
    # clang-tidy will misinterpret them.
    options = options.replace("-std=c++0x ", "")
    options = options.replace("-std=c++14 ", "")

    # Add -DNDEBUG so that editors show the correct size information for structs.
    options += " -DNDEBUG"

    if args.vscode:
        # Visual Studio Code doesn't seem to like "-iquote". Replace it with
        # old-style "-I".
        options = options.replace("-iquote ", "-I ")

    if args.ignore_system_headers:
        # Remove all include options for /usr/* directories
        options = RE_INCLUDE_SYSTEM.sub("", options)

    if isHeader(target["file"]):
        options += " -Wno-pragma-once-outside-header -Wno-unused-const-variable"
        options += " -Wno-unused-function"
        if not target["file"].startswith("external/"):
            # *.h file is treated as C header by default while our headers files are all C++17.
            options = "-x c++ -std=c++17 -fexceptions " + options

    target["command"] = " ".join([cc, options])
    return target


def fixCompilationDatabase(args, db):
    db = [
        modifyCompileCommand(target, args)
        for target in db
        if isCompileTarget(target, args)
    ]

    with open("compile_commands.json", "w") as db_file:
        json.dump(db, db_file, indent=2)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Generate JSON compilation database"
    )
    parser.add_argument("--include_external", action="store_true")
    parser.add_argument("--include_genfiles", action="store_true")
    parser.add_argument("--include_headers", action="store_true")
    parser.add_argument("--vscode", action="store_true")
    parser.add_argument("--ignore_system_headers", action="store_true")
    parser.add_argument("--dedup_targets", action="store_true")
    parser.add_argument("bazel_targets", nargs="*", default=["//..."])
    args = parser.parse_args()
    fixCompilationDatabase(args, generateCompilationDatabase(args))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/key_value/file_based/v3/config.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/key_value/file_based/v3/config.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n5envoy/extensions/key_value/file_based/v3/config.proto\x12(envoy.extensions.key_value.file_based.v3\x1a\x1egoogle/protobuf/duration.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xa9\x01\n\x1c\x46ileBasedKeyValueStoreConfig\x12\x19\n\x08\x66ilename\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x31\n\x0e\x66lush_interval\x18\x02 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x31\n\x0bmax_entries\x18\x03 \x01(\x0b\x32\x1c.google.protobuf.UInt32Value:\x08\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x42\xad\x01\n6io.envoyproxy.envoy.extensions.key_value.file_based.v3B\x0b\x43onfigProtoP\x01Z\\github.com/envoyproxy/go-control-plane/envoy/extensions/key_value/file_based/v3;file_basedv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.key_value.file_based.v3.config_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n6io.envoyproxy.envoy.extensions.key_value.file_based.v3B\013ConfigProtoP\001Z\\github.com/envoyproxy/go-control-plane/envoy/extensions/key_value/file_based/v3;file_basedv3\272\200\310\321\006\002\020\002'
  _globals['_FILEBASEDKEYVALUESTORECONFIG'].fields_by_name['filename']._loaded_options = None
  _globals['_FILEBASEDKEYVALUESTORECONFIG'].fields_by_name['filename']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_FILEBASEDKEYVALUESTORECONFIG']._loaded_options = None
  _globals['_FILEBASEDKEYVALUESTORECONFIG']._serialized_options = b'\322\306\244\341\006\002\010\001'
  _globals['_FILEBASEDKEYVALUESTORECONFIG']._serialized_start=253
  _globals['_FILEBASEDKEYVALUESTORECONFIG']._serialized_end=422
# @@protoc_insertion_point(module_scope)
